spool 11_pk_DWH.log

prompt creando packages pk_DWH...

CREATE OR REPLACE PACKAGE pk_DWH AS
/******************************************************************************
   NOMBRE:   pk_DWH
   DESCRIPCIÓN: PAQUETE DE PROCEDIMIENTOS DE ESTADISTICAS DE LA BBDD

******************************************************************************/

  /**************************************
   *  CONSULTA 1
   **************************************/
  PROCEDURE DWH_PARTICIPACION_CONVOCATORIA (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 2
   **************************************/
  PROCEDURE DWH_PARTICIPACION_MUNICIPIO (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 3
   **************************************/
  PROCEDURE DWH_PARTICIPACION_TELEMATICA (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 4
   **************************************/
  PROCEDURE DWH_VARIACION_VOTOS (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 5
   **************************************/
  PROCEDURE DWH_FIJOS_MESA (
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 6
   **************************************/
  PROCEDURE DWH_ALEGACION_MESA (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 7
   **************************************/
  PROCEDURE DWH_PARTIDOS_PRESENTACION (
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 8
   **************************************/
  PROCEDURE DWH_SORTEO_MESAS (
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 9
   **************************************/
  PROCEDURE DWH_MESAS_MENOS_VOTOS (
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 10
   **************************************/
  PROCEDURE DWH_NOTIFICACION_MESA (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 11
   **************************************/
  PROCEDURE DWH_MAX_CANDIDATOS (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2);

  /**************************************
   *  CONSULTA 12
   **************************************/
  PROCEDURE DWH_REPARTO_ESCANOS (
	RSP OUT VARCHAR2);

END pk_DWH;
/
CREATE OR REPLACE PACKAGE BODY pk_DWH AS

  /******************************************************************************
   NOMBRE:      CONSULTA 1
   ******************************************************************************/
   PROCEDURE DWH_PARTICIPACION_CONVOCATORIA (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   PARAM_IN      VARCHAR2(200);
   EXISTE		 VARCHAR2(1);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   cVotantes	 NUMBER;
   cVotos		 NUMBER;
   cPorcentaje   NUMBER;
   cNulo		 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;

     --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
     END;
     
	 -- Se obtiene el número de votantes de una demarcación.
     BEGIN
       SELECT SUM(VOTANTES)
         INTO cVotantes
         FROM DEMARCACION
        WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
		cVotantes := 0;
     END;

	 -- Se obtiene el número de votos de una demarcación.
     BEGIN
       SELECT SUM(VOTOS)
         INTO cVotos
         FROM DEMARCACION D, RESULTADOS R
        WHERE D.ID_DEMARCACION = R.ID_DEMARCACION
			AND D.ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
		cVotos := 0;
     END;

	 -- Se obtiene el porcentaje de participación existente en las estadisticas.
     BEGIN
       SELECT PORCENT_PARTICIPACION
         INTO cPorcentaje
         FROM DWH_PARTICIPACION_CONVOCATORIA;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
		cPorcentaje := 0;
		cNulo := 0;
     END;

     IF ROUND((cVotos * 100) / cVotantes, 2) > cPorcentaje OR cNulo = 0 THEN
		DELETE DWH_PARTICIPACION_CONVOCATORIA;

  	    INSERT INTO DWH_PARTICIPACION_CONVOCATORIA(ID_CONVOCATORIA, PORCENT_PARTICIPACION)
        VALUES (pIdconvocatoria, ROUND((cVotos * 100) / cVotantes, 2));
		RSP := 'OK';
	 ELSE
		RSP := 'ERROR: La participación de la convocatoria es inferior a la ya registrada.';
     END IF;
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_PARTICIPACION_CONVOCATORIA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_DWH.DWH_PARTICIPACION_CONVOCATORIA', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
	  RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_PARTICIPACION_CONVOCATORIA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END DWH_PARTICIPACION_CONVOCATORIA;

  /******************************************************************************
   NOMBRE:      CONSULTA 2
   ******************************************************************************/
   PROCEDURE DWH_PARTICIPACION_MUNICIPIO (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   PARAM_IN      VARCHAR2(200);
   EXISTE		 VARCHAR2(1);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   cVotantes	 NUMBER;
   cVotos		 NUMBER;
   cPorcentaje   NUMBER;
   cNulo		 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;

     --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
     END;
     
	FOR fMunicipio IN (SELECT CV.ID_MUNICIPIO
						FROM DEMARCACION D, CENTRO_VOTACION CV
						WHERE D.ID_DEMARCACION = CV.ID_DEMARCACION
							AND D.ID_CONVOCATORIA = pIdconvocatoria)
	LOOP
		-- Se obtiene el numero de votos emitidos en cada municipio
		SELECT SUM(R.VOTOS)
			INTO cVotos
		FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D, RECUENTO R
		WHERE D.ID_DEMARCACION = CV.ID_DEMARCACION
			AND CV.ID_DEMARCACION = M.ID_DEMARCACION
			AND CV.ID_MUNICIPIO = M.ID_MUNICIPIO
			AND M.ID_MESA = R.ID_MESA
			AND M.ESTADO = 'FINALIZADA'
			AND D.ID_CONVOCATORIA = pIdconvocatoria
			AND CV.ID_MUNICIPIO = fMunicipio.ID_MUNICIPIO;

		-- Se obtiene el numero de votantes en cada municipio
		SELECT COUNT(*)
			INTO cVotantes
		FROM censo_electoral CE, MESA M
		WHERE ce.id_mesa = m.id_mesa
			AND CE.ID_CONVOCATORIA = pIdconvocatoria
			AND M.ID_MUNICIPIO = fMunicipio.ID_MUNICIPIO;

		-- Se obtiene el porcentaje existente, en caso de que no exista se asigna cero.
		BEGIN
		SELECT PORCENT_MUNICIPIO
			INTO cPorcentaje
        FROM DWH_PARTICIPACION_MUNICIPIO;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				cPorcentaje := 0;
				cNulo := 0;
		END;

     IF ROUND((cVotos * 100) / cVotantes, 2) > cPorcentaje OR cNulo = 0 THEN
		DELETE DWH_PARTICIPACION_MUNICIPIO;

  	    INSERT INTO DWH_PARTICIPACION_MUNICIPIO(ID_CONVOCATORIA, ID_MUNICIPIO, PORCENT_MUNICIPIO)
			VALUES (pIdconvocatoria, fMunicipio.ID_MUNICIPIO, ROUND((cVotos * 100) / cVotantes, 2));
		RSP := 'OK';
	 ELSE
		RSP := 'ERROR: La participación municipal es inferior a la ya registrada.';
     END IF;
	END LOOP;
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_PARTICIPACION_MUNICIPIO', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_DWH.DWH_PARTICIPACION_MUNICIPIO', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_PARTICIPACION_MUNICIPIO', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END DWH_PARTICIPACION_MUNICIPIO;

  /******************************************************************************
   NOMBRE:      CONSULTA 3
   ******************************************************************************/
   PROCEDURE DWH_PARTICIPACION_TELEMATICA (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   PARAM_IN      VARCHAR2(200);
   EXISTE		 VARCHAR2(1);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   ano_conv		 VARCHAR2(4);
   cVotantes	 NUMBER;
   cVotos		 NUMBER;
   cPorcentaje   NUMBER;
   cNulo		 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;

     --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
     END;

    SELECT TO_CHAR(FECHA_INICIO, 'YYYY')
		INTO ano_conv
	FROM CONVOCATORIA
	WHERE ID_CONVOCATORIA = pIdconvocatoria;
 
	FOR fConvocatoria IN (SELECT ID_CONVOCATORIA
						  FROM CONVOCATORIA
						  WHERE TO_CHAR(FECHA_INICIO, 'YYYY') = ano_conv
							AND ESTADO = 'FINALIZADA')
	LOOP
		BEGIN
			SELECT SUM(VOTANTES)
				INTO cVotantes
			FROM DEMARCACION
			WHERE ID_CONVOCATORIA = fConvocatoria.ID_CONVOCATORIA;
			EXCEPTION
				WHEN NO_DATA_FOUND THEN
				cVotantes := 0;
		END;

		BEGIN
			SELECT COUNT(*)
				INTO cVotos
			FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D, VOTACION V
			WHERE D.ID_DEMARCACION = CV.ID_DEMARCACION
				AND CV.ID_DEMARCACION = M.ID_DEMARCACION
				AND CV.ID_MUNICIPIO = M.ID_MUNICIPIO
				AND M.ID_MESA = V.ID_MESA
				AND V.TIPO = 'TELEMATICO'
				AND D.ID_CONVOCATORIA = fConvocatoria.ID_CONVOCATORIA;
			EXCEPTION
				WHEN NO_DATA_FOUND THEN
				cVotos := 0;
		END;

		-- Se obtiene el porcentaje existente, en caso de que no exista se asigna cero.
		BEGIN
		SELECT PORCENTAJE_VOTOS
			INTO cPorcentaje
        FROM DWH_PARTICIPACION_TELEMATICA;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				cPorcentaje := 0;
				cNulo := 0;
		END;

     IF ROUND((cVotos * 100) / cVotantes, 2) > cPorcentaje OR cNulo = 0 THEN
		DELETE DWH_PARTICIPACION_TELEMATICA;

  	    INSERT INTO DWH_PARTICIPACION_TELEMATICA(ANO_CONVOCATORIA, PORCENTAJE_VOTOS)
        VALUES (ano_conv, ROUND((cVotos * 100) / cVotantes, 2));
		RSP := 'OK';
	 ELSE
		RSP := 'ERROR: La participación telematica es inferior a la ya registrada.';
     END IF;
	END LOOP;
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_PARTICIPACION_TELEMATICA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_DWH.DWH_PARTICIPACION_TELEMATICA', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_PARTICIPACION_TELEMATICA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END DWH_PARTICIPACION_TELEMATICA;

  /******************************************************************************
   NOMBRE:      CONSULTA 4
   ******************************************************************************/
   PROCEDURE DWH_VARIACION_VOTOS (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   PARAM_IN      VARCHAR2(200);
   EXISTE		 VARCHAR2(1);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   cConvocatoria VARCHAR2(20);
   cVotosIni	 NUMBER;
   cVotosFin	 NUMBER;
   cVariacion	 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;

     --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
     END;
	
	-- Se obtiene la convocatoria inmediatamente anterior finalizada.
	SELECT ID_CONVOCATORIA
		INTO cConvocatoria
	FROM CONVOCATORIA
	WHERE ESTADO = 'FINALIZADA'
		AND ID_CONVOCATORIA <> pIdconvocatoria
		AND FECHA_FIN = (SELECT max(FECHA_FIN) 
						FROM CONVOCATORIA
						WHERE ESTADO = 'FINALIZADA'
							AND ID_CONVOCATORIA <> pIdconvocatoria);	
	
	-- Se obtienen los votos de la convocatoria inmediatamente anterior, en caso de que no exista se asigna cero.
     BEGIN
       SELECT SUM(VOTOS)
         INTO cVotosIni
         FROM DEMARCACION D, RESULTADOS R
        WHERE D.ID_DEMARCACION = R.ID_DEMARCACION
			AND D.ID_CONVOCATORIA = cConvocatoria;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
		cVotosIni := 0;
     END;

	-- Se obtienen los votos de la convocatoria actual, en caso de que no exista se asigna cero.
     BEGIN
       SELECT SUM(VOTOS)
         INTO cVotosFin
         FROM DEMARCACION D, RESULTADOS R
        WHERE D.ID_DEMARCACION = R.ID_DEMARCACION
			AND D.ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
		cVotosFin := 0;
     END;
	 
	 -- Calculamos la variacion
	 cVariacion := ROUND((100 * (cVotosFin - cVotosIni)) / cVotosIni, 2);
	 
	DELETE FROM DWH_VARIACION_VOTOS;
		
	INSERT INTO DWH_VARIACION_VOTOS(ID_CONV_INI, ID_CONV_FIN, PORCENT_VARIACION)
		VALUES(cConvocatoria, pIdconvocatoria, cVariacion);
	RSP := 'OK';
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_VARIACION_VOTOS', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_DWH.DWH_VARIACION_VOTOS', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_VARIACION_VOTOS', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END DWH_VARIACION_VOTOS;

  /******************************************************************************
   NOMBRE:      CONSULTA 5
   ******************************************************************************/
   PROCEDURE DWH_FIJOS_MESA (
	RSP OUT VARCHAR2) IS

   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   cPersonas	 NUMBER;

   BEGIN
	
	-- Se obtiene el numero de personas que han participado 2 o mas veces en una mesa electoral.
	SELECT COUNT(DISTINCT ID_PADRON)
		INTO cPersonas
	FROM CENSO_ELECTORAL
	WHERE ID_PADRON IN (SELECT ID_PADRON
						FROM CENSO_ELECTORAL
						WHERE PRESENTADO = 'S'
						GROUP BY ID_PADRON
						HAVING COUNT(*) > 1);

	DELETE FROM DWH_FIJOS_MESA;
		
	INSERT INTO DWH_FIJOS_MESA(PERSONAS)
		VALUES(cPersonas);
	RSP := 'OK';
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_FIJOS_MESA', 'SIN PARAMETROS', 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_FIJOS_MESA', 'SIN PARAMETROS', 'RSP: '||RSP);
      COMMIT;

  END DWH_FIJOS_MESA;

  /******************************************************************************
   NOMBRE:      CONSULTA 6
   ******************************************************************************/
   PROCEDURE DWH_ALEGACION_MESA (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   PARAM_IN      VARCHAR2(200);
   EXISTE		 VARCHAR2(1);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   cMiembros	 NUMBER;
   cAlega		 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;

     --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
     END;
	
	-- Se obtiene el numero de personas que han sido seleccionados para formar parte de la mesa electoral.
	SELECT COUNT (ID_PADRON)
		INTO cMiembros
	FROM CENSO_ELECTORAL
	WHERE ID_CONVOCATORIA = pIdconvocatoria
		AND ES_MESA = 'S';

	-- Se obtiene el numero de personas que no han sido localizadas para notificarles.
	SELECT COUNT (ID_PADRON)
		INTO cAlega
	FROM CENSO_ELECTORAL
	WHERE ID_CONVOCATORIA = pIdconvocatoria
		AND ALEGA = 'S';
		
	IF cMiembros = 0 THEN
		DELETE FROM DWH_ALEGACION_MESA;
		
		INSERT INTO DWH_ALEGACION_MESA(ID_CONVOCATORIA, PORCENT_ALEGACION)
			VALUES(pIdconvocatoria, 0);
		RSP := 'OK';
	ELSE
		DELETE FROM DWH_ALEGACION_MESA;
		
		INSERT INTO DWH_ALEGACION_MESA(ID_CONVOCATORIA, PORCENT_ALEGACION)
			VALUES(pIdconvocatoria, ROUND((cAlega * 100) / cMiembros, 2));
		RSP := 'OK';
	END IF;
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_ALEGACION_MESA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_DWH.DWH_ALEGACION_MESA', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_ALEGACION_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END DWH_ALEGACION_MESA;

  /******************************************************************************
   NOMBRE:      CONSULTA 7
   ******************************************************************************/
   PROCEDURE DWH_PARTIDOS_PRESENTACION (
	RSP OUT VARCHAR2) IS

   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   cDemarcacion	 NUMBER;
   cPartido		 NUMBER;
   cPresentado	 NUMBER;
   control		 NUMBER;

   BEGIN
	-- Comenzamos eliminando los datos registrados en la tabla.
	DELETE FROM DWH_PARTIDOS_PRESENTACION;
	
	FOR fPartido IN (SELECT ID_PARTIDO
					 FROM PARTIDO_POLITICO)
	LOOP
		control := 0;
		FOR fConvocatoria IN (SELECT *
							  FROM CONVOCATORIA
							  WHERE ESTADO = 'FINALIZADA'
								AND FECHA_FIN >= ADD_MONTHS(SYSDATE,-120))
		LOOP
			cPartido := 0;
			-- Se obtiene el numero de demarcaciones de cada convocatoria.
			SELECT COUNT(*)
				INTO cDemarcacion
			FROM DEMARCACION
			WHERE ID_CONVOCATORIA = fConvocatoria.ID_CONVOCATORIA;

			FOR fDemarcacion IN (SELECT *
								 FROM DEMARCACION
								 WHERE ID_CONVOCATORIA = fConvocatoria.ID_CONVOCATORIA)
			LOOP
				SELECT COUNT(*)
					INTO cPresentado
				FROM CANDIDATURA
				WHERE ID_DEMARCACION = fDemarcacion.ID_DEMARCACION
					AND ID_PARTIDO = fPartido.ID_PARTIDO
					AND ESTADO = 'APROBADA';
				
				cPartido := cPartido + cPresentado;
			END LOOP;

			IF cPartido = cDemarcacion THEN
				control := 1;
			ELSE
				control := 0;
			END IF;
			EXIT WHEN control = 0;
		END LOOP;
		
		IF control = 1 THEN
			INSERT INTO DWH_PARTIDOS_PRESENTACION(ID_PARTIDO)
				VALUES (fPartido.ID_PARTIDO);
		END IF;
		RSP := 'OK';
	END LOOP;
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_PARTIDOS_PRESENTACION', 'SIN PARAMETROS', 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_PARTIDOS_PRESENTACION', 'SIN PARAMETROS', 'RSP: '||RSP);
      COMMIT;

  END DWH_PARTIDOS_PRESENTACION;

  /******************************************************************************
   NOMBRE:      CONSULTA 8
   ******************************************************************************/
   PROCEDURE DWH_SORTEO_MESAS (
	RSP OUT VARCHAR2) IS

   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   cSorteado	 NUMBER;
   cMunicipio	 NUMBER;

   BEGIN
	
	FOR fConvocatoria IN (SELECT ID_CONVOCATORIA
						  FROM CONVOCATORIA)
	LOOP
		-- Se obtiene el numero total de municipios que participan en la convocatoria.
		SELECT COUNT (DISTINCT CV.ID_MUNICIPIO)
			INTO cMunicipio
		FROM CENTRO_VOTACION CV, DEMARCACION D
		WHERE CV.ID_DEMARCACION = D.ID_DEMARCACION
			AND D.ID_CONVOCATORIA = fConvocatoria.ID_CONVOCATORIA;

		-- Se obtiene el numero total de municipios que no han realizado el sorteo de mesas.
		SELECT COUNT (DISTINCT CV.ID_MUNICIPIO)
			INTO cSorteado
		FROM CENTRO_VOTACION CV, DEMARCACION D
		WHERE CV.ID_DEMARCACION = D.ID_DEMARCACION
			AND CV.SORTEADO = 'N'
			AND D.ID_CONVOCATORIA = fConvocatoria.ID_CONVOCATORIA;

		DELETE FROM DWH_SORTEO_MESAS;
		
		IF cMunicipio = 0 THEN
			INSERT INTO DWH_SORTEO_MESAS(ID_CONVOCATORIA, PORCENT_MUNICIPIO)
				VALUES(fConvocatoria.ID_CONVOCATORIA, 0);
		ELSE
			INSERT INTO DWH_SORTEO_MESAS(ID_CONVOCATORIA, PORCENT_MUNICIPIO)
				VALUES(fConvocatoria.ID_CONVOCATORIA, ROUND((cSorteado * 100) / cMunicipio, 2));
			RSP := 'OK';
		END IF;
	END LOOP;
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_SORTEO_MESAS', 'SIN PARAMETROS', 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_SORTEO_MESAS', 'SIN PARAMETROS', 'RSP: '||RSP);
      COMMIT;

  END DWH_SORTEO_MESAS;

  /******************************************************************************
   NOMBRE:      CONSULTA 9
   ******************************************************************************/
   PROCEDURE DWH_MESAS_MENOS_VOTOS (
	RSP OUT VARCHAR2) IS

   err_num       NUMBER;
   err_msg       VARCHAR2(200);

   BEGIN

	DELETE FROM DWH_MESAS_MENOS_VOTOS;
	INSERT INTO DWH_MESAS_MENOS_VOTOS(ID_CONVOCATORIA, ID_MESA, NUM_VOTOS)
		SELECT * FROM (SELECT D.ID_CONVOCATORIA, M.ID_MESA, COUNT(*)
						FROM MESA M, VOTACION V, CENTRO_VOTACION CV, DEMARCACION D
						WHERE M.ID_MESA = V.ID_MESA
							AND CV.ID_DEMARCACION = M.ID_DEMARCACION
							AND CV.ID_MUNICIPIO = M.ID_MUNICIPIO
							AND CV.ID_DEMARCACION = D.ID_DEMARCACION
						GROUP BY D.ID_CONVOCATORIA, M.ID_MESA
						ORDER BY COUNT(*))
		WHERE ROWNUM <=20;

	RSP := 'OK';
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_MESAS_MENOS_VOTOS', 'SIN PARAMETROS', 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_MESAS_MENOS_VOTOS', 'SIN PARAMETROS', 'RSP: '||RSP);
      COMMIT;

  END DWH_MESAS_MENOS_VOTOS;

  /******************************************************************************
   NOMBRE:      CONSULTA 10
   ******************************************************************************/
   PROCEDURE DWH_NOTIFICACION_MESA (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   PARAM_IN      VARCHAR2(200);
   EXISTE		 VARCHAR2(1);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   cMiembros	 NUMBER;
   cNotific		 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;

     --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
     END;
	
	-- Se obtiene el numero de personas que han sido seleccionados para formar parte de la mesa electoral.
	SELECT COUNT (ID_PADRON)
		INTO cMiembros
	FROM CENSO_ELECTORAL
	WHERE ID_CONVOCATORIA = pIdconvocatoria
		AND ES_MESA = 'S';

	-- Se obtiene el numero de personas que no han sido localizadas para notificarles.
	SELECT COUNT (ID_PADRON)
		INTO cNotific
	FROM CENSO_ELECTORAL
	WHERE ID_CONVOCATORIA = pIdconvocatoria
		AND NOTIFICADO = 'N';
	
	IF cMiembros = 0 THEN
		DELETE FROM DWH_NOTIFICACION_MESA;
	
		INSERT INTO DWH_NOTIFICACION_MESA(ID_CONVOCATORIA, PORCENTAJE_NOTIF)
			VALUES(pIdconvocatoria, 0);
		RSP := 'OK';
	ELSE
		DELETE FROM DWH_NOTIFICACION_MESA;
	
		INSERT INTO DWH_NOTIFICACION_MESA(ID_CONVOCATORIA, PORCENTAJE_NOTIF)
			VALUES(pIdconvocatoria, ROUND((cNotific * 100) / cMiembros, 2));
		RSP := 'OK';
	END IF;
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_NOTIFICACION_MESA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_DWH.DWH_NOTIFICACION_MESA', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_NOTIFICACION_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END DWH_NOTIFICACION_MESA;

  /******************************************************************************
   NOMBRE:      CONSULTA 11
   ******************************************************************************/
   PROCEDURE DWH_MAX_CANDIDATOS (
	pIdconvocatoria IN VARCHAR2,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   PARAM_IN      VARCHAR2(200);
   EXISTE		 VARCHAR2(1);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   cPadron		 VARCHAR2(20);
   cCandidatos	 NUMBER;
   cNulo		 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;

     --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
     END;
	
	SELECT COUNT (C.ID_PADRON)
		INTO cPadron
	FROM CANDIDATO C, CANDIDATURA CA
	WHERE CA.ID_DEMARCACION = C.ID_DEMARCACION
		AND CA.ID_PARTIDO = C.ID_PARTIDO
		AND CA.ESTADO = 'APROBADA'
		AND C.ID_CONVOCATORIA = pIdconvocatoria;

	-- Se obtienen los candidatos existentes, en caso de que no exista se asigna cero.
	BEGIN
	SELECT NUM_CANDIDATOS 
		INTO cCandidatos
	FROM DWH_MAX_CANDIDATOS;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			cCandidatos := 0;
			cNulo := 0;
	END;
	
	IF cPadron > cCandidatos OR cNulo = 0 THEN
		DELETE FROM DWH_MAX_CANDIDATOS;
		
		INSERT INTO DWH_MAX_CANDIDATOS(ID_CONVOCATORIA, NUM_CANDIDATOS)
			VALUES(pIdconvocatoria, cPadron);
		RSP := 'OK';
	ELSE
		RSP := 'ERROR: El numero de candidatos es inferior a los que ya se encuentran registrados.';
	END IF;
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_MAX_CANDIDATOS', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_DWH.DWH_MAX_CANDIDATOS', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_MAX_CANDIDATOS', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END DWH_MAX_CANDIDATOS;

  /******************************************************************************
   NOMBRE:      CONSULTA 12
   ******************************************************************************/
   PROCEDURE DWH_REPARTO_ESCANOS (
	RSP OUT VARCHAR2) IS

   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   cConvocatoria VARCHAR2(20);
   cConv2		 VARCHAR2(20);
   cNulo		 NUMBER;

   BEGIN
	
	SELECT ID_CONVOCATORIA
		INTO cConvocatoria
	FROM CONVOCATORIA
	WHERE ESTADO = 'FINALIZADA'
		AND FECHA_FIN = (SELECT max(FECHA_FIN) 
						FROM CONVOCATORIA
						WHERE ESTADO = 'FINALIZADA');	
	
	-- Se obtiene la convocatoria existente, en caso de que no exista se asigna cero.
	BEGIN
	SELECT DISTINCT ID_CONVOCATORIA 
		INTO cConv2
	FROM DWH_REPARTO_ESCANOS;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			cConv2 := 0;
			cNulo := 0;
	END;
	
	IF cConvocatoria <> cConv2 OR cNulo = 0 THEN
		DELETE FROM DWH_REPARTO_ESCANOS;
		
		INSERT INTO DWH_REPARTO_ESCANOS(ID_CONVOCATORIA, ID_PARTIDO, NUM_ESCANOS)
			SELECT D.ID_CONVOCATORIA, R.PARTIDO, SUM(R.ESCANOS)
			FROM DEMARCACION D, RESULTADOS R
			WHERE D.ID_DEMARCACION = R.ID_DEMARCACION
				AND D.ID_CONVOCATORIA = cConvocatoria
			GROUP BY D.ID_CONVOCATORIA, R.PARTIDO;
		RSP := 'OK';
	ELSE
		RSP := 'ERROR: Los datos de la última convocatoria ya se encuentran registrados.';
	END IF;
	 
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
     VALUES(SYSDATE,'PK_DWH.DWH_REPARTO_ESCANOS', 'SIN PARAMETROS', 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_DWH.DWH_REPARTO_ESCANOS', 'SIN PARAMETROS', 'RSP: '||RSP);
      COMMIT;

  END DWH_REPARTO_ESCANOS;

END pk_DWH;
