spool 12_pk_PRC.log

prompt creando packages pk_PRC...

CREATE OR REPLACE PACKAGE pk_PRC AS
/******************************************************************************
   NOMBRE:   pk_PRC
   DESCRIPCIoN: PAQUETE DE PROCEDIMIENTOS DE FUNCIONALIDAD DE LA BBDD

******************************************************************************/

  /**************************************
   *  REGISTAR VOTACION
   **************************************/
  PROCEDURE ALTA_VOTACION (
	pTipo IN VOTACION.TIPO%TYPE,
	pCertificado IN VOTACION.CERTIFICADO%TYPE,
	pIdmesa IN VOTACION.ID_MESA%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  MANTENIMIENTO RECUENTO
   **************************************/
  PROCEDURE ALTA_RECUENTO (
	pIdpartido IN RECUENTO.PARTIDO%TYPE,
	pVotos IN RECUENTO.VOTOS%TYPE,
	pIdmesa IN RECUENTO.ID_MESA%TYPE,
	RSP OUT VARCHAR2);

  PROCEDURE BAJA_RECUENTO (
	pIdpartido IN RECUENTO.PARTIDO%TYPE,
	pIdmesa IN RECUENTO.ID_MESA%TYPE,
	RSP OUT VARCHAR2);

  PROCEDURE MODIFICACION_RECUENTO (
	pIdpartido IN RECUENTO.PARTIDO%TYPE,
	pVotos IN RECUENTO.VOTOS%TYPE,
	pIdmesa IN RECUENTO.ID_MESA%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  GENERAR_DEMARCACION
   **************************************/
  PROCEDURE GENERAR_DEMARCACION (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  GENERAR_CENSO_ELECTORAL
   **************************************/
  PROCEDURE GENERAR_CENSO_ELECTORAL (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  SORTEO_MIEMBROS_MESA
   **************************************/
  PROCEDURE SORTEO_MIEMBROS_MESA (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	pIddemarcacion IN DEMARCACION.ID_DEMARCACION%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  ENVIAR_NOTIFICACION_MESA
   **************************************/
  PROCEDURE ENVIAR_NOTIFICACION_MESA (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	pIdmunicipio IN CENTRO_VOTACION.ID_MUNICIPIO%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  VALIDAR_CANDIDATURA
   **************************************/
   PROCEDURE VALIDAR_CANDIDATURA (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	pIddemarcacion IN CANDIDATURA.ID_DEMARCACION%TYPE,
	pIdpartido IN CANDIDATURA.ID_PARTIDO%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  CIERRE_CANDIDATURAS
   **************************************/
   PROCEDURE CIERRE_CANDIDATURAS (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	pIddemarcacion IN CANDIDATURA.ID_DEMARCACION%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  CONFIRMAR_NOTIFICACION_MESA
   **************************************/
  PROCEDURE CONFIRMAR_NOTIFICACION_MESA (
	pIdconvocatoria IN CENSO_ELECTORAL.ID_CONVOCATORIA%TYPE,
	pIdpadron IN CENSO_ELECTORAL.ID_PADRON%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  ALEGA_ASISTENCIA_MESA
   **************************************/
  PROCEDURE ALEGA_ASISTENCIA_MESA (
	pIdconvocatoria IN CENSO_ELECTORAL.ID_CONVOCATORIA%TYPE,
	pIdpadron IN CENSO_ELECTORAL.ID_PADRON%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  ABRIR_MESA
   **************************************/
   PROCEDURE ABRIR_MESA (
	pIddemarcacion IN MESA.ID_DEMARCACION%TYPE,
	pIdmunicipio IN MESA.ID_MUNICIPIO%TYPE,
	pIdmesa IN MESA.ID_MESA%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  CIERRE_MESA
   **************************************/
   PROCEDURE CIERRE_MESA (
	pIddemarcacion IN MESA.ID_DEMARCACION%TYPE,
	pIdmunicipio IN MESA.ID_MUNICIPIO%TYPE,
	pIdmesa IN MESA.ID_MESA%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  FINALIZAR_MESA
   **************************************/
   PROCEDURE FINALIZAR_MESA (
	pIddemarcacion IN MESA.ID_DEMARCACION%TYPE,
	pIdmunicipio IN MESA.ID_MUNICIPIO%TYPE,
	pIdmesa IN MESA.ID_MESA%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  PRESENTACION_MIEMBRO_MESA
   **************************************/
   PROCEDURE PRESENTACION_MIEMBRO_MESA (
	pIdconvocatoria IN CENSO_ELECTORAL.ID_CONVOCATORIA%TYPE,
	pIdpadron IN CENSO_ELECTORAL.ID_PADRON%TYPE,
	pIdmesa IN CENSO_ELECTORAL.ID_MESA%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  CIERRE_CONVOCATORIA
   **************************************/
   PROCEDURE CIERRE_CONVOCATORIA (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  FINALIZAR_CONVOCATORIA
   **************************************/
   PROCEDURE FINALIZAR_CONVOCATORIA (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	RSP OUT VARCHAR2);

  /**************************************
   *  REALIZAR_ESCRUTINIO
   **************************************/
  PROCEDURE REALIZAR_ESCRUTINIO (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	pIddemarcacion IN DEMARCACION.ID_DEMARCACION%TYPE,
	RSP OUT VARCHAR2);



END pk_PRC;
/
CREATE OR REPLACE PACKAGE BODY pk_PRC AS

  /******************************************************************************
   NOMBRE:      ALTA_VOTACION
   ******************************************************************************/
   PROCEDURE ALTA_VOTACION (
	pTipo IN VOTACION.TIPO%TYPE,
	pCertificado IN VOTACION.CERTIFICADO%TYPE,
	pIdmesa IN VOTACION.ID_MESA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_votos		 NUMBER;
   c_votantes	 NUMBER;

   BEGIN
     PARAM_IN := 'TIPO: '||pTipo||', CERTIFICADO: '||pCertificado||', ID_MESA: '||pIdmesa;
     
	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El tipo de voto no puede ser nulo.';
     IF pTipo IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El valor del tipo de voto es incorrecto. Valores correctos: PRESENCIAL, TELEMATICO' ;
     IF pTipo NOT IN ('PRESENCIAL', 'TELEMATICO')  THEN  RAISE E_ERROR; END IF;

     RSP := 'ERROR: El certificado no puede ser nulo si el voto es Telematico.';
     IF pTipo = 'TELEMATICO' AND pCertificado IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la mesa no puede ser nulo.';
     IF pIdmesa IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la mesa de votacion existe
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
        RSP := 'ERROR: La mesa no existe.';
        RAISE E_ERROR;
     END;

     --Se verifica que la mesa se encuentra Abierta
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa
		AND ESTADO = 'ABIERTA';
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
        RSP := 'ERROR: La mesa no se encuentra Abierta para poder votar.';
        RAISE E_ERROR;
     END;

	--Se obtiene el numero de posibles votantes
	SELECT COUNT(*)
		INTO c_votantes
	FROM CENSO_ELECTORAL CE, MESA M
	WHERE CE.ID_MESA = M.ID_MESA
		AND M.ID_MESA = pIdmesa;
		
	SELECT COUNT(*)
		INTO c_votos
	FROM MESA M, VOTACION V
	WHERE M.ID_MESA = V.ID_MESA
		AND M.ID_MESA = pIdmesa;

	IF c_votos < c_votantes THEN
		-- Se inserta registro en la tabla
		INSERT INTO VOTACION (TIPO, CERTIFICADO, ID_MESA)
			VALUES (pTipo, pCertificado, pIdmesa);
		RSP := 'OK';
	ELSE
		RSP := 'ERROR: En esta mesa no se pueden registrar mas votos.';
	END IF;

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.ALTA_VOTACION', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.ALTA_VOTACION', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.ALTA_VOTACION', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END ALTA_VOTACION;

  /******************************************************************************
   NOMBRE:      ALTA_RECUENTO
   ******************************************************************************/
   PROCEDURE ALTA_RECUENTO (
	pIdpartido IN RECUENTO.PARTIDO%TYPE,
	pVotos IN RECUENTO.VOTOS%TYPE,
	pIdmesa IN RECUENTO.ID_MESA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_votantes	 NUMBER;

   BEGIN
     PARAM_IN := 'PARTIDO: '||pIdpartido||', VOTOS: '||pVotos||', ID_MESA: '||pIdmesa;
     
	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo del partido politico no puede ser nulo.';
     IF pIdpartido IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El numero de votos no puede ser nulo o menor que CERO.';
     IF pVotos IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la mesa no puede ser nulo.';
     IF pIdmesa IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la mesa de votacion existe
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
        RSP := 'ERROR: La mesa no existe.';
        RAISE E_ERROR;
     END;

     --Se verifica que el partido politico exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM PARTIDO_POLITICO
       WHERE ID_PARTIDO = pIdpartido;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El partido politico no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que el partido politico tienen representacion en la mesa
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CANDIDATURA C, MESA M
       WHERE C.ID_DEMARCACION = M.ID_DEMARCACION
		AND C.ID_PARTIDO = pIdpartido
		AND M.ID_MESA = pIdmesa
		AND C.ESTADO = 'APROBADA';
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El partido politico no tiene representacion en esta mesa.';
           RAISE E_ERROR;
      END;

     --Se verifica que la mesa se encuentre Cerrada
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa
			AND ESTADO = 'CERRADA';
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
        RSP := 'ERROR: La mesa no se encuentra cerrada para poder realizar el recuento.';
        RAISE E_ERROR;
     END;

     --Se verifica que el registro no se haya insertado previamente
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM RECUENTO
       WHERE ID_MESA = pIdmesa
		  AND PARTIDO = pIdpartido
          AND ROWNUM  = 1;
        RSP := 'ERROR: El recuento ya ha sido dado de alta.';
        RAISE E_ERROR;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            NULL;
     END;
	
	--Se verifica que el numero de votos no exceda del numero de votantes
	SELECT COUNT(*)
		INTO c_votantes
	FROM CENSO_ELECTORAL CE, MESA M
	WHERE CE.ID_MESA = M.ID_MESA
		AND M.ID_MESA = pIdmesa;
	
	IF pVotos > c_votantes THEN
		RSP := 'ERROR: El numero de votos indicados excede en el numero de votantes de la mesa.';
		RAISE E_ERROR;
	END IF;

     -- Se inserta registro en la tabla
     INSERT INTO RECUENTO (PARTIDO, VOTOS, ID_MESA)
       VALUES (pIdpartido, pVotos, pIdmesa);
     RSP := 'OK';

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.ALTA_RECUENTO', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.ALTA_RECUENTO', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.ALTA_RECUENTO', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END ALTA_RECUENTO;

  /******************************************************************************
   NOMBRE:      BAJA_RECUENTO
  ******************************************************************************/
  PROCEDURE BAJA_RECUENTO (
	pIdpartido IN RECUENTO.PARTIDO%TYPE,
	pIdmesa IN RECUENTO.ID_MESA%TYPE,
	RSP OUT VARCHAR2) IS
   
   E_ERROR EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);

   BEGIN
     PARAM_IN := 'PARTIDO: '||pIdpartido||', ID_MESA: '||pIdmesa;

     --Se verifica que alguno de los parametros tenga valor
     RSP := 'ERROR: El codigo del partido politico no puede ser nulo.';
     IF pIdpartido IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la mesa no puede ser nulo.';
     IF pIdmesa IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que el recuento existe en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM RECUENTO
       WHERE ID_MESA = pIdmesa 
		  AND PARTIDO = pIdpartido;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El recuento no existe.';
           RAISE E_ERROR;
     END;

     ----Se verifica que la mesa se encuentre Cerrada
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa
			AND ESTADO = 'CERRADA';
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
        RSP := 'ERROR: El recuento no se puede eliminar si la mesa se encuentra finalizada.';
        RAISE E_ERROR;
     END;

     --Se elimina el recuento.
     BEGIN
        DELETE RECUENTO 
		WHERE ID_MESA = pIdmesa 
		  AND PARTIDO = pIdpartido;
     EXCEPTION
       WHEN OTHERS THEN
			RSP := SUBSTR(SQLERRM,1,200);
			RAISE E_ERROR;
     END;
     RSP := 'OK';

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE, 'PK_PRC.BAJA_RECUENTO', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

   EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_PRC.BAJA_RECUENTO', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

     WHEN OTHERS THEN
	   err_num := SQLCODE;
       err_msg := SUBSTR(SQLERRM, 1, 200);
       RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

	   INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
		 VALUES(SYSDATE, 'PK_PRC.BAJA_RECUENTO', PARAM_IN, 'RSP: '||RSP);
       COMMIT;

  END BAJA_RECUENTO;
  
  /******************************************************************************
   NOMBRE:      MODIFICACION_RECUENTO
  ******************************************************************************/
  PROCEDURE MODIFICACION_RECUENTO (
	pIdpartido IN RECUENTO.PARTIDO%TYPE,
	pVotos IN RECUENTO.VOTOS%TYPE,
	pIdmesa IN RECUENTO.ID_MESA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_votantes	 NUMBER;

   BEGIN
     PARAM_IN := 'PARTIDO: '||pIdpartido||', VOTOS: '||pVotos||', ID_MESA: '||pIdmesa;
     
	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo del partido politico no puede ser nulo.';
     IF pIdpartido IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El numero de votos no puede ser nulo o menor que CERO.';
     IF pVotos IS NULL OR pVotos < 0 THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la mesa no puede ser nulo.';
     IF pIdmesa IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que el recuento exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM RECUENTO
       WHERE ID_MESA = pIdmesa
		  AND PARTIDO = pIdpartido;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
        RSP := 'ERROR: El recuento no existe.';
        RAISE E_ERROR;
     END;

     ----Se verifica que la mesa se encuentre Cerrada
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa
			AND ESTADO = 'CERRADA';
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
        RSP := 'ERROR: El recuento no se puede actualizar si la mesa no esta cerrada.';
        RAISE E_ERROR;
     END;

	--Se verifica que el numero de votos no exceda del numero de votantes
	SELECT COUNT(*)
		INTO c_votantes
	FROM CENSO_ELECTORAL CE, MESA M
	WHERE CE.ID_MESA = M.ID_MESA
		AND M.ID_MESA = pIdmesa;
	
	IF pVotos > c_votantes THEN
		RSP := 'ERROR: El numero de votos indicados excede en el numero de votantes de la mesa.';
		RAISE E_ERROR;
	END IF;

     --Si se pasan las pruebas de validacion actualiza
	BEGIN
    UPDATE RECUENTO 
		SET VOTOS = pVotos
    WHERE ID_MESA = pIdmesa 
		AND PARTIDO = pIdpartido;
	EXCEPTION
		WHEN OTHERS THEN
		RSP := SUBSTR(SQLERRM,1,200);
		RAISE E_ERROR;
	END;
	RSP := 'OK';

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE, 'PK_PRC.MODIFICACION_RECUENTO', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

   EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_PRC.MODIFICACION_RECUENTO', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

     WHEN OTHERS THEN
	   err_num := SQLCODE;
       err_msg := SUBSTR(SQLERRM, 1, 200);
       RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

	   INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
		 VALUES(SYSDATE, 'PK_PRC.MODIFICACION_RECUENTO', PARAM_IN, 'RSP: '||RSP);
       COMMIT;

  END MODIFICACION_RECUENTO;

  /******************************************************************************
   NOMBRE:      GENERAR_DEMARCACION
   ******************************************************************************/
   PROCEDURE GENERAR_DEMARCACION (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       	EXCEPTION;
   existe        	VARCHAR2(1);
   PARAM_IN      	VARCHAR2(200);
   err_num       	NUMBER;
   err_msg       	VARCHAR2(200);
   RSP1		       	VARCHAR2(200);
   c_fechafin		DATE;
   c_edad			NUMBER;
   c_paxmesa		NUMBER;
   c_votantes		NUMBER;
   c_provincias		NUMBER;
   c_mesas			NUMBER;
   i_mesa           NUMBER;
   
   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;
	 
	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la convocatoria este abierta
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ESTADO = 'ABIERTA'
          AND ROWNUM  = 1;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
		  RSP := 'ERROR: La convocatoria no se encuentra ABIERTA, por lo que no se pueden generar las demarcaciones.';
          RAISE E_ERROR;
     END;

     --Se verifica que la convocatoria no tenga demarcaciones creadas.
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM DEMARCACION
       WHERE ID_CONVOCATORIA = pIdconvocatoria
          AND ROWNUM  = 1;
       RSP := 'ERROR: La convocatoria ya tiene las demarcaciones creadas.';
       RAISE E_ERROR;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            NULL;
     END;

	-- obtener los datos de una convocatoria
	FOR fConvocatoria IN (SELECT * FROM CONVOCATORIA WHERE ID_CONVOCATORIA = pIdconvocatoria)
	LOOP
		--Se obtiene la fecha de fin y la edad minima.
		SELECT C.FECHA_FIN, P.VALOR
			INTO c_fechafin, c_edad
		FROM CONVOCATORIA C
		INNER JOIN CONVOCATORIA_PARAMETROS CP
			ON c.ID_CONVOCATORIA = CP.ID_CONVOCATORIA
		INNER JOIN PARAMETROS P
			ON CP.ID_PARAMETRO = P.ID_PARAMETRO
			AND P.TIPO = 'CENSO'
			AND P.CAMPO = 'EDAD'
		WHERE C.ID_CONVOCATORIA = pIdconvocatoria;

		--Se obtiene el numero de personas por mesa electoral.
		SELECT P.VALOR
			INTO c_paxmesa
		FROM CONVOCATORIA C
		INNER JOIN CONVOCATORIA_PARAMETROS CP
			ON c.ID_CONVOCATORIA = CP.ID_CONVOCATORIA
		INNER JOIN PARAMETROS P
			ON CP.ID_PARAMETRO = P.ID_PARAMETRO
			AND P.TIPO = 'CENSO'
			AND P.CAMPO = 'PAXMESA'
		WHERE C.ID_CONVOCATORIA = pIdconvocatoria;

		-- Se comprueba el ambito de la convocatoria para filtrar como crear las demarcaciones.
		IF fConvocatoria.TIPO = 'MUNICIPAL' THEN
			FOR fMunicipio IN (SELECT * FROM MUNICIPIO)
			LOOP
				--Se obtiene el numero de votantes para cada municipio.
				SELECT COUNT(*)
					INTO c_votantes
				FROM PADRON_MUNICIPAL P
				WHERE (MONTHS_BETWEEN(c_fechafin, P.FECHA_NACIMIENTO)/12) >= c_edad
					AND P.MUNICIPIO_DOMICILIO = fMunicipio.ID_MUNICIPIO;
				
				-- No se crea demarcacion si el municipio no tiene votantes.
				IF c_votantes <> 0 THEN
					-- Inicializo el contador de generacion de mesas
					i_mesa := 1;

					-- Inicializo el numero de mesas necesarias
					c_mesas := 1;
					
					INSERT INTO DEMARCACION (ESCANOS, VOTANTES, ID_CONVOCATORIA)
						VALUES (fConvocatoria.NUMERO_ESCANOS, c_votantes, pIdconvocatoria);
					
					-- Creamos el centro de votacion
					INSERT INTO CENTRO_VOTACION (ID_DEMARCACION, ID_MUNICIPIO, DISTRITO, SECCION, DIRECCION, CODIGO_POSTAL)
						VALUES (SEC_DEMARCACION.CURRVAL, fMunicipio.ID_MUNICIPIO, TRUNC(DBMS_RANDOM.VALUE(1,1001)), 
								TRUNC(DBMS_RANDOM.VALUE(1,1001)),  DBMS_RANDOM.string('U', 20), TRUNC(DBMS_RANDOM.VALUE(1,100001)));

                    -- Calculamos cual es el numero de mesas que vamos a necesitar en funcion de los votantes del municipio.
                    -- Creamos uno por el numero de personas indicado en el parametro definido en la convocatoria
                    c_mesas:= CEIL(c_votantes/c_paxmesa);
                    
					WHILE (i_mesa <= c_mesas)
					LOOP
						INSERT INTO MESA (DESCRIPCION, ESTADO, ID_DEMARCACION, ID_MUNICIPIO)
							VALUES ('Municipales. Mesa: '||i_mesa||', Municipio: '||fMunicipio.DESCRIPCION, 'CERRADA', SEC_DEMARCACION.CURRVAL, fMunicipio.ID_MUNICIPIO);
						RSP := 'OK';
                        
                        i_mesa := i_mesa + 1;
					END LOOP;
				END IF;
			END LOOP;
		END IF;
		IF fConvocatoria.TIPO = 'AUTONOMICA' THEN
			FOR fProvinciaAutonimia IN (SELECT * FROM PROVINCIA WHERE ID_AUTONOMIA = fConvocatoria.AUTONOMIA)
			LOOP
				--Se obtiene el numero de votantes para cada provincia.
				SELECT COUNT(*)
					INTO c_votantes
				FROM PADRON_MUNICIPAL P
				WHERE (MONTHS_BETWEEN(c_fechafin, P.FECHA_NACIMIENTO)/12) >= c_edad
					AND P.PROVINCIA_DOMICILIO = fProvinciaAutonimia.ID_PROVINCIA;
				
				-- No se crea demarcacion si el municipio no tiene votantes.
				IF c_votantes <> 0 THEN
					INSERT INTO DEMARCACION (ESCANOS, VOTANTES, ID_CONVOCATORIA)
						VALUES (fConvocatoria.NUMERO_ESCANOS, c_votantes, pIdconvocatoria);

					-- Recorremos todos los municipios de la provincia para crear los centros.
					FOR fMunicipio IN (SELECT * FROM MUNICIPIO WHERE ID_PROVINCIA = fProvinciaAutonimia.ID_PROVINCIA)
					LOOP
						-- Inicializo el numero de mesas necesarias
						c_mesas := 1;

						--Se obtiene el numero de votantes para municipio.
						SELECT COUNT(*)
							INTO c_votantes
						FROM PADRON_MUNICIPAL P
						WHERE (MONTHS_BETWEEN(c_fechafin, P.FECHA_NACIMIENTO)/12) >= c_edad
							AND P.MUNICIPIO_DOMICILIO = fMunicipio.ID_MUNICIPIO;

						-- No se crea demarcacion si el municipio no tiene votantes.
						IF c_votantes <> 0 THEN
							-- Inicializo el contador de generacion de mesas
							i_mesa := 1;

							-- Creamos el centro de votacion
							INSERT INTO CENTRO_VOTACION (ID_DEMARCACION, ID_MUNICIPIO, DISTRITO, SECCION, DIRECCION, CODIGO_POSTAL)
								VALUES (SEC_DEMARCACION.CURRVAL, fMunicipio.ID_MUNICIPIO, TRUNC(DBMS_RANDOM.VALUE(1,1001)), 
										TRUNC(DBMS_RANDOM.VALUE(1,1001)),  DBMS_RANDOM.string('U', 20), TRUNC(DBMS_RANDOM.VALUE(1,100001)));

							-- Calculamos cual es el numero de mesas que vamos a necesitar en funcion de los votantes del municipio.
							-- Creamos uno por el numero de personas indicado en el parametro definido en la convocatoria
							c_mesas:= CEIL(c_votantes/c_paxmesa);
                    
							WHILE (i_mesa <= c_mesas)
							LOOP
								INSERT INTO MESA (DESCRIPCION, ESTADO, ID_DEMARCACION, ID_MUNICIPIO)
									VALUES ('Autonomicas. Mesa: '||i_mesa||', Municipio: '||fMunicipio.DESCRIPCION, 'CERRADA', SEC_DEMARCACION.CURRVAL, fMunicipio.ID_MUNICIPIO);
								RSP := 'OK';
                        
								i_mesa := i_mesa + 1;
							END LOOP;
						END IF;
					END LOOP;
				END IF;
			END LOOP;
		END IF;
		IF fConvocatoria.TIPO = 'ESTATAL' THEN
			FOR fProvincia IN (SELECT * FROM PROVINCIA)
			LOOP
				--Se obtiene el numero de votantes para cada provincia.
				SELECT COUNT(*)
					INTO c_votantes
				FROM PADRON_MUNICIPAL P
				WHERE (MONTHS_BETWEEN(c_fechafin, P.FECHA_NACIMIENTO)/12) >= c_edad
					AND P.PROVINCIA_DOMICILIO = fProvincia.ID_PROVINCIA;
				
				-- No se crea demarcacion si la provincia no tiene votantes.
				IF c_votantes <> 0 THEN
					INSERT INTO DEMARCACION (ESCANOS, VOTANTES, ID_CONVOCATORIA)
						VALUES (fConvocatoria.NUMERO_ESCANOS, c_votantes, pIdconvocatoria);

					-- Recorremos todos los municipios de la provincia para crear los centros.
					FOR fMunicipio IN (SELECT * FROM MUNICIPIO WHERE ID_PROVINCIA = fProvincia.ID_PROVINCIA)
					LOOP
						-- Inicializo el numero de mesas necesarias
						c_mesas := 1;

						--Se obtiene el numero de votantes para municipio.
						SELECT COUNT(*)
							INTO c_votantes
						FROM PADRON_MUNICIPAL P
						WHERE (MONTHS_BETWEEN(c_fechafin, P.FECHA_NACIMIENTO)/12) >= c_edad
							AND P.MUNICIPIO_DOMICILIO = fMunicipio.ID_MUNICIPIO;

						-- No se crea demarcacion si el municipio no tiene votantes.
						IF c_votantes <> 0 THEN
							-- Inicializo el contador de generacion de mesas
							i_mesa := 1;

							-- Creamos el centro de votacion
							INSERT INTO CENTRO_VOTACION (ID_DEMARCACION, ID_MUNICIPIO, DISTRITO, SECCION, DIRECCION, CODIGO_POSTAL)
								VALUES (SEC_DEMARCACION.CURRVAL, fMunicipio.ID_MUNICIPIO, TRUNC(DBMS_RANDOM.VALUE(1,1001)), 
										TRUNC(DBMS_RANDOM.VALUE(1,1001)),  DBMS_RANDOM.string('U', 20), TRUNC(DBMS_RANDOM.VALUE(1,100001)));

							-- Calculamos cual es el numero de mesas que vamos a necesitar en funcion de los votantes del municipio.
							-- Creamos uno por el numero de personas indicado en el parametro definido en la convocatoria
							c_mesas:= CEIL(c_votantes/c_paxmesa);
                    
							WHILE (i_mesa <= c_mesas)
							LOOP
								INSERT INTO MESA (DESCRIPCION, ESTADO, ID_DEMARCACION, ID_MUNICIPIO)
									VALUES ('Estatales. Mesa: '||i_mesa||', Municipio: '||fMunicipio.DESCRIPCION, 'CERRADA', SEC_DEMARCACION.CURRVAL, fMunicipio.ID_MUNICIPIO);
								RSP := 'OK';
                        
								i_mesa := i_mesa + 1;
							END LOOP;
						END IF;
					END LOOP;
				END IF;
			END LOOP;
		END IF;
	END LOOP;
	
	-- Se actualizan los datos para la consulta 8.
	PK_DWH.DWH_SORTEO_MESAS (RSP1);
	
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.GENERAR_DEMARCACION', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.GENERAR_DEMARCACION', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.GENERAR_DEMARCACION', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END GENERAR_DEMARCACION;

  /******************************************************************************
   NOMBRE:      GENERAR_CENSO_ELECTORAL
   ******************************************************************************/
   PROCEDURE GENERAR_CENSO_ELECTORAL (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_fechafin	 DATE;
   c_edad		 NUMBER;
   c_paxmesa	 NUMBER;
   c_tipo		 VARCHAR2(10);
   c_numcenso	 NUMBER;
   c_autonomia	 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;
	 
	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la convocatoria este abierta
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ESTADO = 'ABIERTA'
          AND ROWNUM  = 1;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
		  RSP := 'ERROR: La convocatoria no se encuentra ABIERTA, por lo que no se pueden generar el censo electoral.';
          RAISE E_ERROR;
     END;

     --Se verifica que el censo no haya sido generado
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
          AND ROWNUM  = 1;
		  RSP := 'ERROR: La convocatoria ya tiene generado el censo electoral.';
          RAISE E_ERROR;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
	   NULL;
     END;

	--Se obtiene el tipo de convocatoria, autonomia, la fecha de fin y la edad minima para poder votar.
	SELECT C.TIPO, C.AUTONOMIA, C.FECHA_FIN, P.VALOR
		INTO c_tipo, c_autonomia, c_fechafin, c_edad
	FROM CONVOCATORIA C
	INNER JOIN CONVOCATORIA_PARAMETROS CP
		ON c.ID_CONVOCATORIA = CP.ID_CONVOCATORIA
	INNER JOIN parametros P
		ON CP.ID_PARAMETRO = P.ID_PARAMETRO
		AND P.TIPO = 'CENSO'
		AND P.CAMPO = 'EDAD'
	WHERE C.ID_CONVOCATORIA = pIdconvocatoria;

	--Se obtiene el numero de personas por mesa electoral.
	SELECT P.VALOR
		INTO c_paxmesa
	FROM CONVOCATORIA C
	INNER JOIN CONVOCATORIA_PARAMETROS CP
		ON c.ID_CONVOCATORIA = CP.ID_CONVOCATORIA
	INNER JOIN parametros P
		ON CP.ID_PARAMETRO = P.ID_PARAMETRO
		AND P.TIPO = 'CENSO'
		AND P.CAMPO = 'PAXMESA'
	WHERE C.ID_CONVOCATORIA = pIdconvocatoria;

	-- Se comprueba el ambito de la convocatoria para filtrar como crear el censo.
	IF c_tipo = 'MUNICIPAL' THEN
		-- Obtenemos la lista de municipios a cargar
		FOR fMunicipio IN (SELECT * FROM MUNICIPIO)
		LOOP
			-- Obtenemos la lista de mesas a asignar
			FOR fMesa IN (SELECT M.ID_MESA, M.DESCRIPCION, M.ESTADO
						  FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D
						  WHERE M.ID_DEMARCACION = CV.ID_DEMARCACION
							AND M.ID_MUNICIPIO = CV.ID_MUNICIPIO
							AND D.ID_DEMARCACION = CV.ID_DEMARCACION
							AND D.ID_CONVOCATORIA = pIdconvocatoria
							AND CV.ID_MUNICIPIO = fMunicipio.ID_MUNICIPIO)
			LOOP
				c_numcenso := 0;
				-- Obtenemos la lista de personas a incluir
				FOR fPadron IN (SELECT * FROM PADRON_MUNICIPAL 
								WHERE (MONTHS_BETWEEN(c_fechafin, FECHA_NACIMIENTO)/12) >= c_edad
								AND MUNICIPIO_DOMICILIO = fMunicipio.ID_MUNICIPIO)
				LOOP
				     --Se verifica que la persona del censo no se encuentra ya asignada a una mesa.
					BEGIN
					SELECT 'S'
						INTO EXISTE
					FROM CENSO_ELECTORAL
					WHERE ID_CONVOCATORIA = pIdconvocatoria
						AND ID_PADRON = fPadron.id_padron
						AND ROWNUM  = 1;
					EXCEPTION
					WHEN NO_DATA_FOUND THEN
						EXISTE := 'N';
					END;
					
					IF EXISTE = 'N' THEN
						c_numcenso := c_numcenso + 1;
						INSERT INTO CENSO_ELECTORAL (ID_CONVOCATORIA, ID_PADRON, FECHA_INSCRIPCION, ES_MESA, TIPO, NOTIFICADO, PRESENTADO, ID_MESA)
							VALUES (pIdconvocatoria, fPadron.ID_PADRON, SYSDATE, 'N', 'TITULAR', 'N', 'N', fMesa.ID_MESA);
						RSP := 'OK';
					END IF;

					EXIT WHEN c_numcenso = c_paxmesa;
				END LOOP;
			END LOOP;
		END LOOP;
	END IF;

	-- Se comprueba el ambito de la convocatoria para filtrar como crear el censo.
	IF c_tipo = 'AUTONOMICA' THEN
		-- Obtenemos la lista de municipios a cargar
		FOR fMunicipio IN (SELECT MUN.ID_MUNICIPIO, MUN.DESCRIPCION, MUN.ID_PROVINCIA
							FROM MUNICIPIO MUN
							INNER JOIN PROVINCIA PROV
								ON PROV.ID_PROVINCIA = MUN.ID_PROVINCIA
								AND PROV.ID_AUTONOMIA = c_autonomia)
		LOOP
			-- Obtenemos la lista de mesas a asignar
			FOR fMesa IN (SELECT M.ID_MESA, M.DESCRIPCION, M.ESTADO
						  FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D
						  WHERE M.ID_DEMARCACION = CV.ID_DEMARCACION
							AND M.ID_MUNICIPIO = CV.ID_MUNICIPIO
							AND D.ID_DEMARCACION = CV.ID_DEMARCACION
							AND D.ID_CONVOCATORIA = pIdconvocatoria
							AND CV.ID_MUNICIPIO = fMunicipio.ID_MUNICIPIO)
			LOOP
				c_numcenso := 0;
				-- Obtenemos la lista de personas a incluir
				FOR fPadron IN (SELECT * FROM PADRON_MUNICIPAL 
								WHERE (MONTHS_BETWEEN(c_fechafin, FECHA_NACIMIENTO)/12) >= c_edad
								AND MUNICIPIO_DOMICILIO = fMunicipio.ID_MUNICIPIO)
				LOOP
				     --Se verifica que la persona del censo no se encuentra ya asignada a una mesa.
					BEGIN
					SELECT 'S'
						INTO EXISTE
					FROM CENSO_ELECTORAL
					WHERE ID_CONVOCATORIA = pIdconvocatoria
						AND ID_PADRON = fPadron.id_padron
						AND ROWNUM  = 1;
					EXCEPTION
					WHEN NO_DATA_FOUND THEN
						EXISTE := 'N';
					END;
					
					IF EXISTE = 'N' THEN
						c_numcenso := c_numcenso + 1;
						INSERT INTO CENSO_ELECTORAL (ID_CONVOCATORIA, ID_PADRON, FECHA_INSCRIPCION, ES_MESA, TIPO, NOTIFICADO, PRESENTADO, ID_MESA)
							VALUES (pIdconvocatoria, fPadron.ID_PADRON, SYSDATE, 'N', 'TITULAR', 'N', 'N', fMesa.ID_MESA);
						RSP := 'OK';
					END IF;

					EXIT WHEN c_numcenso = c_paxmesa;
				END LOOP;
			END LOOP;
		END LOOP;
	END IF;

	-- Se comprueba el ambito de la convocatoria para filtrar como crear el censo.
	IF c_tipo = 'ESTATAL' THEN
		-- Obtenemos la lista de municipios a cargar
		FOR fMunicipio IN (SELECT * FROM MUNICIPIO)
		LOOP
			-- Obtenemos la lista de mesas a asignar
			FOR fMesa IN (SELECT M.ID_MESA, M.DESCRIPCION, M.ESTADO
						  FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D
						  WHERE M.ID_DEMARCACION = CV.ID_DEMARCACION
							AND M.ID_MUNICIPIO = CV.ID_MUNICIPIO
							AND D.ID_DEMARCACION = CV.ID_DEMARCACION
							AND D.ID_CONVOCATORIA = pIdconvocatoria
							AND CV.ID_MUNICIPIO = fMunicipio.ID_MUNICIPIO)
			LOOP
				c_numcenso := 0;
				-- Obtenemos la lista de personas a incluir
				FOR fPadron IN (SELECT * FROM PADRON_MUNICIPAL 
								WHERE (MONTHS_BETWEEN(c_fechafin, FECHA_NACIMIENTO)/12) >= c_edad
								AND MUNICIPIO_DOMICILIO = fMunicipio.ID_MUNICIPIO)
				LOOP
					--Se verifica que la persona del censo no se encuentra ya asignada a una mesa.
					BEGIN
					SELECT 'S'
						INTO EXISTE
					FROM CENSO_ELECTORAL
					WHERE ID_CONVOCATORIA = pIdconvocatoria
						AND ID_PADRON = fPadron.id_padron
						AND ROWNUM  = 1;
					EXCEPTION
					WHEN NO_DATA_FOUND THEN
						EXISTE := 'N';
					END;

					IF EXISTE = 'N' THEN
						c_numcenso := c_numcenso + 1;
						INSERT INTO CENSO_ELECTORAL (ID_CONVOCATORIA, ID_PADRON, FECHA_INSCRIPCION, ES_MESA, TIPO, NOTIFICADO, PRESENTADO, ID_MESA)
							VALUES (pIdconvocatoria, fPadron.ID_PADRON, SYSDATE, 'N', 'TITULAR', 'N', 'N', fMesa.ID_MESA);
						RSP := 'OK';
					END IF;
					
					EXIT WHEN c_numcenso = c_paxmesa;
				END LOOP;
			END LOOP;
		END LOOP;
	END IF;

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.GENERAR_CENSO_ELECTORAL', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.GENERAR_CENSO_ELECTORAL', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.GENERAR_CENSO_ELECTORAL', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END GENERAR_CENSO_ELECTORAL;

  /******************************************************************************
   NOMBRE:      SORTEO_MIEMBROS_MESA
   ******************************************************************************/
   PROCEDURE SORTEO_MIEMBROS_MESA (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	pIddemarcacion IN DEMARCACION.ID_DEMARCACION%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   RSP1		     VARCHAR2(200);
   c_fechafin	 DATE;
   c_edad		 NUMBER;
   c_titulo      NUMBER;
   c_idpadron	 NUMBER;
   c_tipo		 VARCHAR2(8);

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria||', ID_DEMARCACION: '||pIddemarcacion;
	 
	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la demarcacion no puede ser nulo.';
     IF pIddemarcacion IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la demarcacion exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM DEMARCACION
       WHERE ID_DEMARCACION = pIddemarcacion;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La demarcacion no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que el censo haya sido generado
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
          AND ROWNUM  = 1;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
		  RSP := 'ERROR: La convocatoria no tiene generado el censo electoral.';
          RAISE E_ERROR;
     END;

     --Se verifica que el sorteo de los componentes de la mesa no haya sido realizado
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENTRO_VOTACION CV, DEMARCACION D 
       WHERE D.ID_DEMARCACION = CV.ID_DEMARCACION
			AND CV.SORTEADO = 'N'
			AND D.ID_CONVOCATORIA = pIdconvocatoria
			AND D.ID_DEMARCACION = pIddemarcacion
			AND ROWNUM  = 1;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
		  RSP := 'ERROR: El sorteo de formacion de mesas ya se encuentra realizado.';
          RAISE E_ERROR;
     END;

	--Se verifica que no exista ninguna candidatura en estado PRESENTADA.
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM DEMARCACION D, CANDIDATURA C
	   WHERE D.ID_DEMARCACION = C.ID_DEMARCACION
			AND D.ID_CONVOCATORIA = pIdconvocatoria
			AND D.ID_DEMARCACION = pIddemarcacion
			AND C.ESTADO = 'PRESENTADA'
			AND ROWNUM  = 1;
        RSP := 'ERROR: Existen candidaturas pendientes de validar.';
        RAISE E_ERROR;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            NULL;
     END;

	--Se obtiene la fecha de fin y la edad minima que debe de tener la persona del censo
	SELECT C.FECHA_FIN, P.VALOR
		INTO c_fechafin, c_edad
	FROM CONVOCATORIA C
	INNER JOIN CONVOCATORIA_PARAMETROS CP
		ON c.ID_CONVOCATORIA = CP.ID_CONVOCATORIA
	INNER JOIN parametros P
		ON CP.ID_PARAMETRO = P.ID_PARAMETRO
		AND P.TIPO = 'MESA'
		AND P.CAMPO = 'EDAD'
	WHERE C.ID_CONVOCATORIA = pIdconvocatoria;

	--Se obtiene el titulo minimo que debe poseer la persona del censo
	SELECT P.VALOR
		INTO c_titulo
	FROM CONVOCATORIA C
	INNER JOIN CONVOCATORIA_PARAMETROS CP
		ON c.ID_CONVOCATORIA = CP.ID_CONVOCATORIA
	INNER JOIN parametros P
		ON CP.ID_PARAMETRO = P.ID_PARAMETRO
		AND P.TIPO = 'MESA'
		AND P.CAMPO = 'TITULACION'
	WHERE C.ID_CONVOCATORIA = pIdconvocatoria;

	-- Obtenemos la lista de mesas a sortear
	FOR fMesa IN (SELECT M.ID_MESA
				  FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D
				  WHERE M.ID_DEMARCACION = CV.ID_DEMARCACION
					AND D.ID_DEMARCACION = CV.ID_DEMARCACION
					AND M.ID_MUNICIPIO = CV.ID_MUNICIPIO
					AND D.ID_CONVOCATORIA = pIdconvocatoria
					AND D.ID_DEMARCACION = pIddemarcacion)
	LOOP
		c_idpadron := 0;
		c_tipo := 'TITULAR';
		-- Obtenemos el id_padron de la lista de personas que cumplen las reglas y no es candidato en una candidatura aprobada.
		FOR fPadron IN (SELECT ID_PADRON 
						FROM CENSO_ELECTORAL
						WHERE ID_CONVOCATORIA = pIdconvocatoria
							AND ID_MESA = fMesa.ID_MESA
							AND ES_MESA = 'N'
							AND ID_PADRON IN (SELECT ID_PADRON
											  FROM PADRON_MUNICIPAL
											  WHERE (MONTHS_BETWEEN(c_fechafin, FECHA_NACIMIENTO)/12) <= c_edad
												AND TITULACION >= c_titulo)
							AND ID_PADRON NOT IN (SELECT C.ID_PADRON
												  FROM CANDIDATO C, CANDIDATURA CA
												  WHERE CA.ID_DEMARCACION = C.ID_DEMARCACION
													AND CA.ID_PARTIDO = C.ID_PARTIDO
													AND CA.ESTADO = 'APROBADA'
													AND C.ID_CONVOCATORIA = pIdconvocatoria)
						ORDER BY DBMS_RANDOM.VALUE)
		LOOP
			c_idpadron := c_idpadron + 1;
			BEGIN
			UPDATE CENSO_ELECTORAL
			SET ES_MESA = 'S', TIPO = c_tipo
			WHERE ID_CONVOCATORIA = pIdconvocatoria
				AND ID_MESA = fMesa.ID_MESA
				AND ID_PADRON = fPadron.ID_PADRON;
			EXCEPTION
				WHEN OTHERS THEN
					RSP := SUBSTR(SQLERRM,1,200);
					RAISE E_ERROR;
			END;
			RSP := 'OK';

			IF c_idpadron = 3 THEN
				c_tipo := 'SUPLENTE';
			END IF;
			EXIT WHEN c_idpadron = 6;
		END LOOP;
	END LOOP;

	BEGIN
	UPDATE CENTRO_VOTACION
	SET SORTEADO = 'S'
	WHERE ID_DEMARCACION = pIddemarcacion;
	EXCEPTION
		WHEN OTHERS THEN
			RSP := SUBSTR(SQLERRM,1,200);
			RAISE E_ERROR;
	END;
	RSP := 'OK';
	
		-- Se actualizan los datos para la consulta 8.
	PK_DWH.DWH_SORTEO_MESAS (RSP1);

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.SORTEO_MIEMBROS_MESA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.SORTEO_MIEMBROS_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.SORTEO_MIEMBROS_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END SORTEO_MIEMBROS_MESA;

  /******************************************************************************
   NOMBRE:      ENVIAR_NOTIFICACION_MESA
   ******************************************************************************/
  PROCEDURE ENVIAR_NOTIFICACION_MESA (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	pIdmunicipio IN CENTRO_VOTACION.ID_MUNICIPIO%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria||', ID_MUNICIPIO: '||pIdmunicipio;
	 
	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo del municipio no puede ser nulo.';
     IF pIdmunicipio IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que el sorteo de los componentes de la mesa haya sido realizado
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENTRO_VOTACION CV, DEMARCACION D 
       WHERE D.ID_DEMARCACION = CV.ID_DEMARCACION
			AND CV.SORTEADO = 'S'
			AND D.ID_CONVOCATORIA = pIdconvocatoria
			AND CV.ID_MUNICIPIO = pIdmunicipio
			AND ROWNUM  = 1;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
		  RSP := 'ERROR: El sorteo de formacion de mesas no se encuentra realizado.';
          RAISE E_ERROR;
     END;

     --Se verifica que la notificacion del sorteo de los componentes de la mesa haya sido realizado
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENTRO_VOTACION CV, DEMARCACION D 
       WHERE D.ID_DEMARCACION = CV.ID_DEMARCACION
			AND CV.NOTIFICADO = 'S'
			AND D.ID_CONVOCATORIA = pIdconvocatoria
			AND CV.ID_MUNICIPIO = pIdmunicipio
			AND ROWNUM  = 1;
			RSP := 'ERROR: La notificacion del sorteo de formacion de mesas ya se encuentra realizado.';
			RAISE E_ERROR;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
			NULL;
     END;

	-- Se listan los datos de todas las personas asignadas con sus datos para la notificacion.
	FOR fMiembros IN (SELECT pm.nombre, pm.apellido1, pm.apellido2, pm.num_documento, ce.tipo, pm.direccion,
					  mun.descripcion as municipio, prov.descripcion as provincia, m.descripcion as mesa, 
					  cv.distrito, cv.seccion, cv.direccion as direccion_centro, cv.codigo_postal, pm.id_padron
					  FROM PADRON_MUNICIPAL PM, CENSO_ELECTORAL CE, MESA M, CENTRO_VOTACION CV, MUNICIPIO MUN, PROVINCIA PROV
					  WHERE PM.ID_PADRON = CE.ID_PADRON
						AND CE.ID_MESA = M.ID_MESA
						AND M.ID_DEMARCACION = CV.ID_DEMARCACION
						AND M.ID_MUNICIPIO = CV.ID_MUNICIPIO
						AND CV.ID_MUNICIPIO = MUN.ID_MUNICIPIO
						AND MUN.ID_PROVINCIA = PROV.ID_PROVINCIA
						AND CE.ES_MESA = 'S'
						AND CE.NOTIFICADO = 'N'
						AND CE.ID_CONVOCATORIA = pIdconvocatoria
						AND CV.ID_MUNICIPIO = pIdmunicipio)
	LOOP
		dbms_output.put_line('DATOS PERSONALES: '||fMiembros.NOMBRE||' '||fMiembros.APELLIDO1||' '||fMiembros.APELLIDO2||' '||fMiembros.NUM_DOCUMENTO);
		dbms_output.put_line(fMiembros.DIRECCION||' '||fMiembros.MUNICIPIO||' '||fMiembros.PROVINCIA);
		dbms_output.put_line('PUESTO ASIGNADO: '||fMiembros.TIPO);
		dbms_output.put_line('DATOS MESA: '||fMiembros.MESA||' '||fMiembros.DISTRITO||' '||fMiembros.SECCION||fMiembros.DIRECCION_CENTRO||' '||fMiembros.CODIGO_POSTAL);
	END LOOP;

	-- Se actualiza el campo Notificado a SI.
	BEGIN
	UPDATE CENTRO_VOTACION
	SET NOTIFICADO = 'S'
	WHERE ID_DEMARCACION IN (SELECT CV.ID_DEMARCACION
							 FROM CENTRO_VOTACION CV, DEMARCACION D 
							 WHERE D.ID_DEMARCACION = CV.ID_DEMARCACION
								AND CV.NOTIFICADO = 'N'
								AND D.ID_CONVOCATORIA = pIdconvocatoria
								AND CV.ID_MUNICIPIO = pIdmunicipio);
	EXCEPTION
		WHEN OTHERS THEN
			RSP := SUBSTR(SQLERRM,1,200);
			RAISE E_ERROR;
	END;
	RSP := 'OK';

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.ENVIAR_NOTIFICACION_MESA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.ENVIAR_NOTIFICACION_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.ENVIAR_NOTIFICACION_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END ENVIAR_NOTIFICACION_MESA;

  /******************************************************************************
   NOMBRE:      VALIDAR_CANDIDATURA
   ******************************************************************************/
   PROCEDURE VALIDAR_CANDIDATURA (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	pIddemarcacion IN CANDIDATURA.ID_DEMARCACION%TYPE,
	pIdpartido IN CANDIDATURA.ID_PARTIDO%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_candidatos  NUMBER;
   c_escanos     NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria||', ID_DEMARCACION: '||pIddemarcacion||', ID_PARTIDO: '||pIdpartido;

	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la demarcacion no puede ser nulo.';
     IF pIddemarcacion IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo del partido politico no puede ser nulo.';
     IF pIdpartido IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la demarcacion exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM DEMARCACION
       WHERE ID_DEMARCACION = pIddemarcacion;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La demarcacion no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que el partido politico exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM PARTIDO_POLITICO
       WHERE ID_PARTIDO = pIdpartido;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El partido politico no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la candidatura exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CANDIDATURA
       WHERE ID_DEMARCACION = pIddemarcacion
		  AND ID_PARTIDO = pIdpartido;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La candidatura no existe.';
           RAISE E_ERROR;
      END;

	--Se obtiene el numero de candidatos que forman la candidatura
	SELECT COUNT(*)
		INTO c_candidatos
	FROM CANDIDATO
	WHERE ID_CONVOCATORIA = pIdconvocatoria
		AND ID_DEMARCACION = pIddemarcacion
		AND ID_PARTIDO = pIdpartido;

	--Se obtiene el numero de escaños que forman la candidatura
	SELECT D.ESCANOS
		INTO c_escanos
	FROM DEMARCACION D, CANDIDATURA C
	WHERE D.ID_DEMARCACION = C.ID_DEMARCACION
		AND D.ID_CONVOCATORIA = pIdconvocatoria
		AND D.ID_DEMARCACION = pIddemarcacion
		AND C.ID_PARTIDO = pIdpartido
		AND C.ESTADO = 'PRESENTADA';

	-- Se realiza la validacion del numero de candidatos necesarios.
	IF c_candidatos >= c_escanos + 5 THEN
		RSP := 'OK';
	ELSE
		RSP := 'ERROR: La candidatura no cumple con el minimo de candidatos necesarios para poder participar en la convocatoria.';
	END IF;

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.VALIDAR_CANDIDATURA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.VALIDAR_CANDIDATURA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.VALIDAR_CANDIDATURA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END VALIDAR_CANDIDATURA;

  /******************************************************************************
   NOMBRE:      CIERRE_CANDIDATURAS
   ******************************************************************************/
   PROCEDURE CIERRE_CANDIDATURAS (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	pIddemarcacion IN CANDIDATURA.ID_DEMARCACION%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_candidatos  NUMBER;
   c_escanos     NUMBER;
   c_estado      VARCHAR2(10);

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria||', ID_DEMARCACION: '||pIddemarcacion;

	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la demarcacion no puede ser nulo.';
     IF pIddemarcacion IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la demarcacion exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM DEMARCACION
       WHERE ID_DEMARCACION = pIddemarcacion;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La demarcacion no existe.';
           RAISE E_ERROR;
      END;

	-- Se listan las candidaturas a validar
	FOR fCandidaturas IN (SELECT c.id_partido
						  FROM DEMARCACION D, CANDIDATURA C
						  WHERE D.ID_DEMARCACION = C.ID_DEMARCACION
							AND D.ID_CONVOCATORIA = pIdconvocatoria
							AND D.ID_DEMARCACION = pIddemarcacion
							AND C.ESTADO = 'PRESENTADA')
	LOOP

		--Se obtiene el numero de candidatos que forman la candidatura
		SELECT COUNT(*)
			INTO c_candidatos
		FROM CANDIDATO
		WHERE ID_CONVOCATORIA = pIdconvocatoria
			AND ID_DEMARCACION = pIddemarcacion
			AND ID_PARTIDO = fCandidaturas.ID_PARTIDO;

		--Se obtiene el numero de candidatos que forman la candidatura
		SELECT D.ESCANOS
			INTO c_escanos
		FROM DEMARCACION D, CANDIDATURA C
		WHERE D.ID_DEMARCACION = C.ID_DEMARCACION
			AND D.ID_CONVOCATORIA = pIdconvocatoria
			AND D.ID_DEMARCACION = pIddemarcacion
			AND C.ID_PARTIDO = fCandidaturas.ID_PARTIDO;

		-- Se realiza la validacion del numero de candidatos necesarios.
		IF c_candidatos >= c_escanos + 5 THEN
			c_estado :='APROBADA';
		ELSE
			c_estado :='RECHAZADA';
		END IF;

		BEGIN
		UPDATE CANDIDATURA 
			SET ESTADO = c_estado
		WHERE ID_DEMARCACION = pIddemarcacion
			AND ID_PARTIDO = fCandidaturas.ID_PARTIDO;
		EXCEPTION
			WHEN OTHERS THEN
				RSP := SUBSTR(SQLERRM,1,200);
				RAISE E_ERROR;
		END;
		RSP := 'OK';

	END LOOP;

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.CIERRE_CANDIDATURAS', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.CIERRE_CANDIDATURAS', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.CIERRE_CANDIDATURAS', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END CIERRE_CANDIDATURAS;

  /******************************************************************************
   NOMBRE:      CONFIRMAR_NOTIFICACION_MESA
   ******************************************************************************/
  PROCEDURE CONFIRMAR_NOTIFICACION_MESA (
	pIdconvocatoria IN CENSO_ELECTORAL.ID_CONVOCATORIA%TYPE,
	pIdpadron IN CENSO_ELECTORAL.ID_PADRON%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_miembros	 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria||', ID_PADRON: '||pIdpadron;

	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo del padron no puede ser nulo.';
     IF pIdpadron IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la persona exista en el censo electoral
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ID_PADRON = pIdpadron;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona no existe en el censo electoral.';
           RAISE E_ERROR;
      END;

     --Se verifica que la persona haya sido designada como miembro de la mesa
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ID_PADRON = pIdpadron
		  AND ES_MESA = 'S';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona no ha sido designada como miembro de la mesa.';
           RAISE E_ERROR;
      END;

     --Se verifica que la notificacion del centro de votacion haya sido realizada.
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL CE, MESA M, CENTRO_VOTACION CV
       WHERE CE.ID_MESA = M.ID_MESA
		  AND M.ID_DEMARCACION = CV.ID_DEMARCACION
		  AND M.ID_MUNICIPIO = CV.ID_MUNICIPIO
		  AND CE.ID_CONVOCATORIA = pIdconvocatoria
		  AND CE.ID_PADRON = pIdpadron
		  AND CV.NOTIFICADO = 'S';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona no ha sido notificada como miembro de la mesa.';
           RAISE E_ERROR;
      END;

	BEGIN
	UPDATE CENSO_ELECTORAL
		SET NOTIFICADO = 'S'
	WHERE ID_CONVOCATORIA = pIdconvocatoria
		AND ID_PADRON = pIdpadron;
	EXCEPTION
		WHEN OTHERS THEN
			RSP := SUBSTR(SQLERRM,1,200);
			RAISE E_ERROR;
	END;
	RSP := 'OK';

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE, 'PK_PRC.CONFIRMAR_NOTIFICACION_MESA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

   EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_PRC.CONFIRMAR_NOTIFICACION_MESA', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

     WHEN OTHERS THEN
	   err_num := SQLCODE;
       err_msg := SUBSTR(SQLERRM, 1, 200);
       RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

	   INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
		 VALUES(SYSDATE, 'PK_PRC.CONFIRMAR_NOTIFICACION_MESA', PARAM_IN, 'RSP: '||RSP);
       COMMIT;

  END CONFIRMAR_NOTIFICACION_MESA;

  /******************************************************************************
   NOMBRE:      ALEGA_ASISTENCIA_MESA
   ******************************************************************************/
  PROCEDURE ALEGA_ASISTENCIA_MESA (
	pIdconvocatoria IN CENSO_ELECTORAL.ID_CONVOCATORIA%TYPE,
	pIdpadron IN CENSO_ELECTORAL.ID_PADRON%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_miembros	 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria||', ID_PADRON: '||pIdpadron;

	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo del padron no puede ser nulo.';
     IF pIdpadron IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la persona exista en el censo electoral
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ID_PADRON = pIdpadron;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona no existe en el censo electoral.';
           RAISE E_ERROR;
      END;

     --Se verifica que la persona haya sido designada como miembro de la mesa
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ID_PADRON = pIdpadron
		  AND ES_MESA = 'S';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona no ha sido designada como miembro de la mesa.';
           RAISE E_ERROR;
      END;

     --Se verifica que la persona haya sido designada como miembro de la mesa
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ID_PADRON = pIdpadron
		  AND ES_MESA = 'S'
		  AND NOTIFICADO = 'S';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona no ha sido notificada como miembro de la mesa.';
           RAISE E_ERROR;
      END;

	BEGIN
	UPDATE CENSO_ELECTORAL
		SET ALEGA = 'S'
	WHERE ID_CONVOCATORIA = pIdconvocatoria
		AND ID_PADRON = pIdpadron;
	EXCEPTION
		WHEN OTHERS THEN
			RSP := SUBSTR(SQLERRM,1,200);
			RAISE E_ERROR;
	END;
	RSP := 'OK';

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE, 'PK_PRC.ALEGA_ASISTENCIA_MESA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

   EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_PRC.ALEGA_ASISTENCIA_MESA', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

     WHEN OTHERS THEN
	   err_num := SQLCODE;
       err_msg := SUBSTR(SQLERRM, 1, 200);
       RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

	   INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
		 VALUES(SYSDATE, 'PK_PRC.ALEGA_ASISTENCIA_MESA', PARAM_IN, 'RSP: '||RSP);
       COMMIT;

  END ALEGA_ASISTENCIA_MESA;

  /******************************************************************************
   NOMBRE:      CIERRE_MESA
   ******************************************************************************/
   PROCEDURE CIERRE_MESA (
	pIddemarcacion IN MESA.ID_DEMARCACION%TYPE,
	pIdmunicipio IN MESA.ID_MUNICIPIO%TYPE,
	pIdmesa IN MESA.ID_MESA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);

   BEGIN
     PARAM_IN := 'ID_DEMARCACION: '||pIddemarcacion||', ID_MUNICIPIO: '||pIdmunicipio||', ID_MESA: '||pIdmesa;

	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la demarcacion no puede ser nulo.';
     IF pIddemarcacion IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo del municipio no puede ser nulo.';
     IF pIdmunicipio IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la mesa no puede ser nulo.';
     IF pIdmesa IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la demarcacion exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM DEMARCACION
       WHERE ID_DEMARCACION = pIddemarcacion;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La demarcacion no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que el municipio existe en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MUNICIPIO
       WHERE ID_MUNICIPIO = pIdmunicipio;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El municipio no existe.';
           RAISE E_ERROR;
     END;

     --Se verifica que la mesa exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La mesa no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la mesa se encuentre abierta
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa
			AND ESTADO = 'ABIERTA';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La mesa no se encuentra ABIERTA.';
           RAISE E_ERROR;
      END;

     --Se verifica que la convocatoria se encuentre abierta
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA C, DEMARCACION D
       WHERE C.ID_CONVOCATORIA = D.ID_CONVOCATORIA
			AND D.ID_DEMARCACION = pIddemarcacion
			AND C.ESTADO = 'ABIERTA';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no se encuentra ABIERTA.';
           RAISE E_ERROR;
      END;
	  
	BEGIN
	UPDATE MESA
		SET ESTADO = 'CERRADA'
	WHERE ID_DEMARCACION = pIddemarcacion
		AND ID_MUNICIPIO = pIdmunicipio
		AND ID_MESA = pIdmesa;
	EXCEPTION
		WHEN OTHERS THEN
			RSP := SUBSTR(SQLERRM,1,200);
			RAISE E_ERROR;
	END;
	RSP := 'OK';

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.CIERRE_MESA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.CIERRE_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.CIERRE_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END CIERRE_MESA;

  /******************************************************************************
   NOMBRE:      FINALIZAR_MESA
   ******************************************************************************/
   PROCEDURE FINALIZAR_MESA (
	pIddemarcacion IN MESA.ID_DEMARCACION%TYPE,
	pIdmunicipio IN MESA.ID_MUNICIPIO%TYPE,
	pIdmesa IN MESA.ID_MESA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_votos	     NUMBER;
   c_recuento    NUMBER;

   BEGIN
     PARAM_IN := 'ID_DEMARCACION: '||pIddemarcacion||', ID_MUNICIPIO: '||pIdmunicipio||', ID_MESA: '||pIdmesa;

	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la demarcacion no puede ser nulo.';
     IF pIddemarcacion IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo del municipio no puede ser nulo.';
     IF pIdmunicipio IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la mesa no puede ser nulo.';
     IF pIdmesa IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la demarcacion exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM DEMARCACION
       WHERE ID_DEMARCACION = pIddemarcacion;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La demarcacion no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que el municipio existe en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MUNICIPIO
       WHERE ID_MUNICIPIO = pIdmunicipio;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El municipio no existe.';
           RAISE E_ERROR;
     END;

     --Se verifica que la mesa exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La mesa no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la mesa se encuentre cerrada
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa
			AND ESTADO = 'CERRADA';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La mesa no se encuentra CERRADA.';
           RAISE E_ERROR;
      END;

     --Se verifica que la convocatoria se encuentre cerrada
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA C, DEMARCACION D
       WHERE C.ID_CONVOCATORIA = D.ID_CONVOCATORIA
			AND D.ID_DEMARCACION = pIddemarcacion
			AND C.ESTADO = 'CERRADA';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no se encuentra CERRADA.';
           RAISE E_ERROR;
      END;

	--Se obtiene el numero de votos emitidos
	SELECT COUNT(V.TIPO)
		INTO c_votos
	FROM MESA M, VOTACION V
	WHERE M.ID_MESA = V.ID_MESA
		AND M.ID_DEMARCACION = pIddemarcacion
		AND M.ID_MUNICIPIO = pIdmunicipio;

	--Se obtiene el numero de votos del recuento realizado
	SELECT SUM(R.VOTOS)
		INTO c_recuento
	FROM MESA M, RECUENTO R
	WHERE M.ID_MESA = R.ID_MESA
		AND M.ID_DEMARCACION = pIddemarcacion
		AND M.ID_MUNICIPIO = pIdmunicipio;
	
	IF c_recuento IS NULL THEN
		c_recuento := 0;
	END IF;
	
	IF c_recuento < c_votos THEN
		RSP := 'ERROR: El numero de votos contados es inferior al numero de votos emitidos.';
	ELSIF c_recuento > c_votos THEN
		RSP := 'ERROR: El numero de votos contados es mayor al numero de votos emitidos.';
	ELSE
		BEGIN
		UPDATE MESA
			SET ESTADO = 'FINALIZADA'
		WHERE ID_DEMARCACION = pIddemarcacion
			AND ID_MUNICIPIO = pIdmunicipio
			AND ID_MESA = pIdmesa;
		EXCEPTION
			WHEN OTHERS THEN
				RSP := SUBSTR(SQLERRM,1,200);
				RAISE E_ERROR;
		END;
		RSP := 'OK';
	END IF;
	  
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.FINALIZAR_MESA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.FINALIZAR_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.FINALIZAR_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END FINALIZAR_MESA;

  /******************************************************************************
   NOMBRE:      ABRIR_MESA
   ******************************************************************************/
   PROCEDURE ABRIR_MESA (
	pIddemarcacion IN MESA.ID_DEMARCACION%TYPE,
	pIdmunicipio IN MESA.ID_MUNICIPIO%TYPE,
	pIdmesa IN MESA.ID_MESA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_miembros	 NUMBER;
   c_convoct     VARCHAR2(200);
   RSP1		     VARCHAR2(200);

   BEGIN
     PARAM_IN := 'ID_DEMARCACION: '||pIddemarcacion||', ID_MUNICIPIO: '||pIdmunicipio||', ID_MESA: '||pIdmesa;

	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la demarcacion no puede ser nulo.';
     IF pIddemarcacion IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo del municipio no puede ser nulo.';
     IF pIdmunicipio IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la mesa no puede ser nulo.';
     IF pIdmesa IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la demarcacion exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM DEMARCACION
       WHERE ID_DEMARCACION = pIddemarcacion;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La demarcacion no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que el municipio existe en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MUNICIPIO
       WHERE ID_MUNICIPIO = pIdmunicipio;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El municipio no existe.';
           RAISE E_ERROR;
     END;

     --Se verifica que la mesa exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La mesa no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la mesa se encuentre cerrada
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa
			AND ESTADO = 'CERRADA';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La mesa no se encuentra CERRADA.';
           RAISE E_ERROR;
      END;

     --Se verifica que la convocatoria se encuentre abierta
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA C, DEMARCACION D
       WHERE C.ID_CONVOCATORIA = D.ID_CONVOCATORIA
			AND D.ID_DEMARCACION = pIddemarcacion
			AND C.ESTADO = 'ABIERTA';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no se encuentra ABIERTA.';
           RAISE E_ERROR;
      END;

	--Se obtiene el codigo de la convocatoria
	SELECT C.ID_CONVOCATORIA
		INTO c_convoct
    FROM CONVOCATORIA C, DEMARCACION D
    WHERE C.ID_CONVOCATORIA = D.ID_CONVOCATORIA
		AND D.ID_DEMARCACION = pIddemarcacion
		AND C.ESTADO = 'ABIERTA';

	--Se obtiene el numero de personas presentadas para conformar la mesa electoral
	SELECT COUNT(CE.PRESENTADO)
		INTO c_miembros
	FROM MESA M, CENSO_ELECTORAL CE
	WHERE M.ID_MESA = CE.ID_MESA
		AND M.ID_DEMARCACION = pIddemarcacion
		AND M.ID_MUNICIPIO = pIdmunicipio
        AND M.ID_MESA = pIdmesa
        AND CE.PRESENTADO = 'S';		
	
	IF c_miembros >= 3 THEN
		BEGIN
		UPDATE MESA
			SET ESTADO = 'ABIERTA'
		WHERE ID_DEMARCACION = pIddemarcacion
			AND ID_MUNICIPIO = pIdmunicipio
			AND ID_MESA = pIdmesa;
		EXCEPTION
			WHEN OTHERS THEN
				RSP := SUBSTR(SQLERRM,1,200);
				RAISE E_ERROR;
		END;
		RSP := 'OK';
	ELSE
		RSP := 'ERROR: La mesa no tiene los miembros necesarios para poder abrirla.';
	END IF;
	
	-- Se actualizan los datos para la consulta 10.
	PK_DWH.DWH_NOTIFICACION_MESA(c_convoct, RSP1);
	
     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.ABRIR_MESA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.ABRIR_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.ABRIR_MESA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END ABRIR_MESA;

  /******************************************************************************
   NOMBRE:      PRESENTACION_MIEMBRO_MESA
  ******************************************************************************/
   PROCEDURE PRESENTACION_MIEMBRO_MESA (
	pIdconvocatoria IN CENSO_ELECTORAL.ID_CONVOCATORIA%TYPE,
	pIdpadron IN CENSO_ELECTORAL.ID_PADRON%TYPE,
	pIdmesa IN CENSO_ELECTORAL.ID_MESA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_miembros	 NUMBER;
   RSP1		     VARCHAR2(200);

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria||', ID_PADRON: '||pIdpadron||', ID_MESA: '||pIdmesa;

	 --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo del padron no puede ser nulo.';
     IF pIdpadron IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la mesa no puede ser nulo.';
     IF pIdmesa IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la persona exista en el censo electoral
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ID_PADRON = pIdpadron;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona no existe en el censo electoral.';
           RAISE E_ERROR;
      END;

     --Se verifica que la mesa exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA
       WHERE ID_MESA = pIdmesa;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La mesa no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que la persona haya sido designada como miembro de la mesa
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ID_PADRON = pIdpadron
		  AND ID_MESA = pIdmesa
		  AND ES_MESA = 'S';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona no ha sido designada como miembro de la mesa.';
           RAISE E_ERROR;
      END;

     --Se verifica que la persona haya sido notificada como miembro de la mesa
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ID_PADRON = pIdpadron
		  AND ID_MESA = pIdmesa
		  AND NOTIFICADO = 'S';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona no ha sido notificada como miembro de la mesa.';
           RAISE E_ERROR;
      END;

     --Se verifica que la persona no se encuentre registrada ya como presentado
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CENSO_ELECTORAL
       WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ID_PADRON = pIdpadron
		  AND ID_MESA = pIdmesa
		  AND PRESENTADO = 'N';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona ya se encuentra registrada como presentado.';
           RAISE E_ERROR;
      END;

	--Se obtiene el numero de personas ya registradas en la mesa
	SELECT COUNT(PRESENTADO)
		INTO c_miembros
	FROM CENSO_ELECTORAL
        WHERE ID_CONVOCATORIA = pIdconvocatoria
		  AND ID_MESA = pIdmesa
        AND PRESENTADO = 'S';		
	
	IF c_miembros < 3 THEN
		BEGIN
		--Si se pasan las pruebas de validacion actualiza
		UPDATE CENSO_ELECTORAL 
			SET PRESENTADO = 'S'
		WHERE ID_CONVOCATORIA = pIdconvocatoria
			AND ID_PADRON = pIdpadron
			AND ID_MESA = pIdmesa;
		EXCEPTION
			WHEN OTHERS THEN
				RSP := SUBSTR(SQLERRM,1,200);
				RAISE E_ERROR;
		END;
		RSP := 'OK';
	ELSE
		RSP := 'ERROR: La mesa ya tiene registrados los miembros necesarios.';
	END IF;

	-- Se actualizan los datos para la consulta 5.
	PK_DWH.DWH_FIJOS_MESA (RSP1);

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE, 'PK_PRC.PRESENTACION_MIEMBRO_MESA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

   EXCEPTION
     WHEN E_ERROR THEN
       INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
         VALUES(SYSDATE, 'PK_PRC.PRESENTACION_MIEMBRO_MESA', PARAM_IN, 'RSP: '||RSP);
	   COMMIT;

     WHEN OTHERS THEN
	   err_num := SQLCODE;
       err_msg := SUBSTR(SQLERRM, 1, 200);
       RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

	   INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
		 VALUES(SYSDATE, 'PK_PRC.PRESENTACION_MIEMBRO_MESA', PARAM_IN, 'RSP: '||RSP);
       COMMIT;

  END PRESENTACION_MIEMBRO_MESA;

  /******************************************************************************
   NOMBRE:      CIERRE_CONVOCATORIA
   ******************************************************************************/
   PROCEDURE CIERRE_CONVOCATORIA (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;

     --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
     END;

     --Se verifica que la convocatoria se encuentre abierta
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria
			AND ESTADO = 'ABIERTA';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no se encuentra abierta.';
           RAISE E_ERROR;
     END;

     --Se verifica que todas las mesas de la convocatoria se encuentren cerradas
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D, CONVOCATORIA C
       WHERE C.ID_CONVOCATORIA = D.ID_CONVOCATORIA
			AND D.ID_DEMARCACION = CV.ID_DEMARCACION
			AND CV.ID_DEMARCACION = M.ID_DEMARCACION
			AND CV.ID_MUNICIPIO = M.ID_MUNICIPIO
			AND M.ESTADO = 'ABIERTA'
			AND C.ID_CONVOCATORIA = pIdconvocatoria
			AND ROWNUM = 1;
            RSP := 'ERROR: La convocatoria no se puede cerrar porque existen mesas abiertas.';
            RAISE E_ERROR;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;

     --Se actualiza el valor del estado a "CERRADA"
     BEGIN
        UPDATE CONVOCATORIA 
		SET ESTADO = 'CERRADA' 
		WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
       WHEN OTHERS THEN
			RSP := SUBSTR(SQLERRM,1,200);
			RAISE E_ERROR;
     END;
     RSP := 'OK';

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.CIERRE_CONVOCATORIA', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.CIERRE_CONVOCATORIA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.CIERRE_CONVOCATORIA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END CIERRE_CONVOCATORIA;

  /******************************************************************************
   NOMBRE:      FINALIZAR_CONVOCATORIA
   ******************************************************************************/
   PROCEDURE FINALIZAR_CONVOCATORIA (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_candidatura NUMBER;
   c_resultado	 NUMBER;
   RSP1		     VARCHAR2(200);

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria;

     --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
     END;

     --Se verifica que la convocatoria se encuentre cerrada
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria
			AND ESTADO = 'CERRADA';
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no se encuentra cerrada.';
           RAISE E_ERROR;
     END;

     --Se verifica que todas las mesas de la convocatoria se encuentren finalizadas
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D, CONVOCATORIA C
       WHERE C.ID_CONVOCATORIA = D.ID_CONVOCATORIA
			AND D.ID_DEMARCACION = CV.ID_DEMARCACION
			AND CV.ID_DEMARCACION = M.ID_DEMARCACION
			AND CV.ID_MUNICIPIO = M.ID_MUNICIPIO
			AND M.ESTADO IN ('ABIERTA', 'CERRADA')
			AND C.ID_CONVOCATORIA = pIdconvocatoria
			AND ROWNUM = 1;
            RSP := 'ERROR: La convocatoria no se puede finalizar porque existen mesas sin finalizar.';
            RAISE E_ERROR;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;

     --Se verifica que el escrutinio se encuentre realizado
	FOR fDemarcacion IN (SELECT C.TIPO, C.AUTONOMIA, D.ID_DEMARCACION
						FROM CONVOCATORIA C, DEMARCACION D
						WHERE C.ID_CONVOCATORIA = D.ID_CONVOCATORIA
							AND C.ID_CONVOCATORIA = pIdconvocatoria)
	LOOP
		-- Se obtiene el numero de candidaturas presentadas en la demarcacion
		SELECT COUNT(*)
			INTO c_candidatura
		FROM DEMARCACION D, CANDIDATURA C
		WHERE D.ID_DEMARCACION = C.ID_DEMARCACION
			AND D.ID_CONVOCATORIA = pIdconvocatoria
			AND D.ID_DEMARCACION = fDemarcacion.ID_DEMARCACION
			AND C.ESTADO = 'APROBADA';

		-- Se obtiene el numero de resultados obtenidos en la demarcacion
		SELECT COUNT(*)
			INTO c_resultado
		FROM DEMARCACION D, RESULTADOS R
		WHERE D.ID_DEMARCACION = R.ID_DEMARCACION
			AND D.ID_CONVOCATORIA = pIdconvocatoria
			AND D.ID_DEMARCACION = fDemarcacion.ID_DEMARCACION;
		
		IF c_candidatura = c_resultado THEN
		    --Se actualiza el valor del estado a "FINALIZADA"
			BEGIN
			UPDATE CONVOCATORIA 
				SET ESTADO = 'FINALIZADA' 
			WHERE ID_CONVOCATORIA = pIdconvocatoria;
			EXCEPTION
			WHEN OTHERS THEN
				RSP := SUBSTR(SQLERRM,1,200);
				RAISE E_ERROR;
			END;
			RSP := 'OK';
		ELSE
            RSP := 'ERROR: Quedan escrutinios pendientes de realizar en la convocatoria.';
            RAISE E_ERROR;
		END IF;
	END LOOP;

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.FINALIZAR_CONVOCATORIA', PARAM_IN, 'RSP: '||RSP);
	 
	 -- Se actualizan los datos estadisticos.
	 PK_DWH.DWH_PARTICIPACION_CONVOCATORIA(pIdconvocatoria, RSP1); -- Consulta 1

	 PK_DWH.DWH_PARTICIPACION_MUNICIPIO(pIdconvocatoria, RSP1); -- Consulta 2

	 PK_DWH.DWH_PARTICIPACION_TELEMATICA(pIdconvocatoria, RSP1); -- Consulta 3

	 PK_DWH.DWH_VARIACION_VOTOS(pIdconvocatoria, RSP1); -- Consulta 4

	 PK_DWH.DWH_PARTIDOS_PRESENTACION(RSP1); -- Consulta 7

	 PK_DWH.DWH_MESAS_MENOS_VOTOS(RSP1); -- Consulta 9

	 PK_DWH.DWH_MAX_CANDIDATOS(pIdconvocatoria, RSP1); -- Consulta 11

	 PK_DWH.DWH_REPARTO_ESCANOS(RSP1); -- Consulta 12
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.FINALIZAR_CONVOCATORIA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.FINALIZAR_CONVOCATORIA', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END FINALIZAR_CONVOCATORIA;

  /******************************************************************************
   NOMBRE:      REALIZAR_ESCRUTINIO
   ******************************************************************************/
  PROCEDURE REALIZAR_ESCRUTINIO (
	pIdconvocatoria IN CONVOCATORIA.ID_CONVOCATORIA%TYPE,
	pIddemarcacion IN DEMARCACION.ID_DEMARCACION%TYPE,
	RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PARAM_IN      VARCHAR2(200);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   c_escrutinio	 VARCHAR(100);
   c_escanos	 NUMBER;
   c_votos		 NUMBER;
   c_partido	 NUMBER;
   c_porvotos	 NUMBER;

   BEGIN
     PARAM_IN := 'ID_CONVOCATORIA: '||pIdconvocatoria||', ID_DEMARCACION: '||pIddemarcacion;

     --Se verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El codigo de la convocatoria no puede ser nulo.';
     IF pIdconvocatoria IS NULL THEN RAISE E_ERROR; END IF;

     RSP := 'ERROR: El codigo de la demarcacion no puede ser nulo.';
     IF pIddemarcacion IS NULL THEN RAISE E_ERROR; END IF;

     --Se verifica que la convocatoria exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM CONVOCATORIA
       WHERE ID_CONVOCATORIA = pIdconvocatoria;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La convocatoria no existe.';
           RAISE E_ERROR;
     END;

     --Se verifica que la demarcacion exista en base de datos
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM DEMARCACION
       WHERE ID_DEMARCACION = pIddemarcacion;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La demarcacion no existe.';
           RAISE E_ERROR;
      END;

     --Se verifica que todas las mesas de la convocatoria y demarcacion se encuentren finalizadas
     BEGIN
       SELECT 'S'
        INTO EXISTE
       FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D
       WHERE D.ID_DEMARCACION = CV.ID_DEMARCACION
			AND CV.ID_DEMARCACION = M.ID_DEMARCACION
			AND CV.ID_MUNICIPIO = M.ID_MUNICIPIO
			AND M.ESTADO IN ('ABIERTA', 'CERRADA')
			AND D.ID_CONVOCATORIA = pIdconvocatoria
			AND D.ID_DEMARCACION = pIddemarcacion
			AND ROWNUM = 1;
            RSP := 'ERROR: Existen mesas sin finalizar en la demarcacion.';
            RAISE E_ERROR;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;

	--Se obtiene el metodo a emplear para realizar el escrutinio.
	SELECT P.VALOR
		INTO c_escrutinio
	FROM CONVOCATORIA C
	INNER JOIN CONVOCATORIA_PARAMETROS CP
		ON c.ID_CONVOCATORIA = CP.ID_CONVOCATORIA
	INNER JOIN parametros P
		ON CP.ID_PARAMETRO = P.ID_PARAMETRO
		AND P.TIPO = 'ESCRUTINIO'
		AND P.CAMPO = 'METODO'
	WHERE C.ID_CONVOCATORIA = pIdconvocatoria;

	--Se obtiene el procentaje minimo de votos que debe de tener un partido para poder participar en el reparto de escaños.
	SELECT P.VALOR
		INTO c_porvotos
	FROM CONVOCATORIA C
	INNER JOIN CONVOCATORIA_PARAMETROS CP
		ON c.ID_CONVOCATORIA = CP.ID_CONVOCATORIA
	INNER JOIN parametros P
		ON CP.ID_PARAMETRO = P.ID_PARAMETRO
		AND P.TIPO = 'ESCRUTINIO'
		AND P.CAMPO = 'PORCVOTOS'
	WHERE C.ID_CONVOCATORIA = pIdconvocatoria;

	-- Se obtiene el numero de escanos a repartir de una convocatoria-demarcacion
	SELECT D.ESCANOS
		INTO c_escanos
	FROM CONVOCATORIA C, DEMARCACION D
	WHERE C.ID_CONVOCATORIA = D.ID_CONVOCATORIA
		AND D.ID_CONVOCATORIA = pIdconvocatoria
		AND D.ID_DEMARCACION = pIddemarcacion;

	-- Se obtiene el numero de votos registrados de una convocatoria-demarcacion
	SELECT SUM(R.VOTOS)
		INTO c_votos
	FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D, RECUENTO R
	WHERE D.ID_DEMARCACION = CV.ID_DEMARCACION
		AND CV.ID_DEMARCACION = M.ID_DEMARCACION
		AND CV.ID_MUNICIPIO = M.ID_MUNICIPIO
		AND M.ID_MESA = R.ID_MESA
		AND M.ESTADO = 'FINALIZADA'
		AND D.ID_CONVOCATORIA = pIdconvocatoria
		AND D.ID_DEMARCACION = pIddemarcacion;
	
	-- Se carga la tabla temporal con los datos de la convocatoria-demarcacion a escrutar
	INSERT INTO TEMP_RESULTADOS (ESCANOS, PARTIDO, VOTOS, VOTO_ESCANO, PORCENTAJE_VOTOS)
		SELECT 0 AS ESCANOS, R.PARTIDO, SUM(R.VOTOS), SUM(R.VOTOS), ROUND((SUM(R.VOTOS) * 100) / c_votos, 2)
		FROM MESA M, CENTRO_VOTACION CV, DEMARCACION D, RECUENTO R
		WHERE D.ID_DEMARCACION = CV.ID_DEMARCACION
			AND CV.ID_DEMARCACION = M.ID_DEMARCACION
			AND CV.ID_MUNICIPIO = M.ID_MUNICIPIO
			AND M.ID_MESA = R.ID_MESA
			AND M.ESTADO = 'FINALIZADA'
			AND D.ID_CONVOCATORIA = pIdconvocatoria
			AND D.ID_DEMARCACION = pIddemarcacion
		GROUP BY 0, R.PARTIDO
		ORDER BY R.PARTIDO;

	IF c_escrutinio = 'DHONDT' THEN
		WHILE c_escanos > 0
		LOOP
			SELECT *
				INTO c_partido
			FROM
				(SELECT PARTIDO
				FROM TEMP_RESULTADOS 
				WHERE VOTO_ESCANO = (SELECT MAX(VOTO_ESCANO)
									FROM TEMP_RESULTADOS
									WHERE PORCENTAJE_VOTOS >= c_porvotos)
					AND PORCENTAJE_VOTOS >= c_porvotos
				ORDER BY ESCANOS)
			WHERE ROWNUM = 1;

			UPDATE TEMP_RESULTADOS
				SET ESCANOS = ESCANOS + 1
			WHERE PARTIDO = c_partido;
		
			UPDATE TEMP_RESULTADOS
				SET VOTO_ESCANO = VOTOS / (ESCANOS + 1)
			WHERE PARTIDO = c_partido;

		c_escanos := c_escanos - 1;
		END LOOP;

		FOR fResultado IN (SELECT * FROM TEMP_RESULTADOS)
		LOOP
			INSERT INTO RESULTADOS (PARTIDO, ESCANOS, VOTOS, PORCENTAJE_VOTOS, ID_DEMARCACION)
				VALUES (fResultado.PARTIDO, fResultado.ESCANOS, fResultado.VOTOS, fResultado.PORCENTAJE_VOTOS, pIddemarcacion);
			RSP := 'OK';
		END LOOP;
	END IF;

	IF c_escrutinio = 'SAINTE-LAGUE' THEN
		WHILE c_escanos > 0
		LOOP
			SELECT *
				INTO c_partido
			FROM
				(SELECT PARTIDO
				FROM TEMP_RESULTADOS 
				WHERE VOTO_ESCANO = (SELECT MAX(VOTO_ESCANO)
									FROM TEMP_RESULTADOS
									WHERE PORCENTAJE_VOTOS >= c_porvotos)
					AND PORCENTAJE_VOTOS >= c_porvotos
				ORDER BY ESCANOS)
			WHERE ROWNUM = 1;

			UPDATE TEMP_RESULTADOS
				SET ESCANOS = ESCANOS + 1
			WHERE PARTIDO = c_partido;
		
			UPDATE TEMP_RESULTADOS
				SET VOTO_ESCANO = VOTOS / (ESCANOS + 2)
			WHERE PARTIDO = c_partido;

		c_escanos := c_escanos - 1;
		END LOOP;

		FOR fResultado IN (SELECT * FROM TEMP_RESULTADOS)
		LOOP
			INSERT INTO RESULTADOS (PARTIDO, ESCANOS, VOTOS, PORCENTAJE_VOTOS, ID_DEMARCACION)
				VALUES (fResultado.PARTIDO, fResultado.ESCANOS, fResultado.VOTOS, fResultado.PORCENTAJE_VOTOS, pIddemarcacion);
			RSP := 'OK';
		END LOOP;
	END IF;

     INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
       VALUES(SYSDATE,'PK_PRC.REALIZAR_ESCRUTINIO', PARAM_IN, 'RSP: '||RSP);
     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN
      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.REALIZAR_ESCRUTINIO', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);
      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOG_PROCESOS (FECHA, PROCEDIMIENTO, PARAMETROS_ENTRADA, PARAMETROS_SALIDA)
        VALUES(SYSDATE, 'PK_PRC.REALIZAR_ESCRUTINIO', PARAM_IN, 'RSP: '||RSP);
      COMMIT;

  END REALIZAR_ESCRUTINIO;

END PK_PRC;
